////////////////////////////////for test
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>

typedef unsigned char uchar;
typedef unsigned int uint;
typedef unsigned long ulong;

uchar Setup(uchar CmPt);
uchar TouchReset( void );
uchar TouchByte(uchar outch);
uchar TouchBit(uchar outbit);

uchar com_port=1;
const uchar TR=1;
const uchar FL=0;

void main(void)
{
    int i;

    printf("setup %d\n",Setup(com_port));
    printf("TouchReset %d\n",TouchReset());

    printf("ReadRom  %d ",TouchBit(1));
    printf(" %d ",TouchBit(1));
    printf(" %d ",TouchBit(0));
    printf(" %d ",TouchBit(0));
    printf(" %d ",TouchBit(1));
    printf(" %d ",TouchBit(1));
    printf(" %d ",TouchBit(0));
    printf(" %d \n\n",TouchBit(0));

    for (i = 0; i < 8; i++)
        printf("%02X ",TouchByte(0xFF));
    printf("\n\n");
}
////////////////////////////////for test



/*
   In the following C language code, 1-wire I/O is accomplished using the
   serial port of an IBM PC or compatible. The serial port must be capable
   of a 115,200 bps data rate. Setup must be called before any of the
   touch functions to verify the existence of the specified com port and
   initialize it.

   --------------------------------------------------------------------
   The setup function makes sure that the com port number passed to it
   is from 1 to 4 and has a valid address associated with it.
*/
uchar Setup(uchar CmPt)
{
     uint far *ptr = (uint far *) 0x00400000;
     uint SPA;

     /* check to see if it is a valid com port number and address */
     SPA = *(ptr+CmPt-1); /* get the address */
     if (CmPt < 1 || CmPt > 4 || !SPA )
        return FL;

     /* serial port initialization */
     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+1,0x00);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA+1,0x00);  /* no interrupts */
     outportb(SPA+4,0x03);  /* RTS and DTR on */

     return TR;
}


/*------------------------------------------------------------------------
 * Do a reset on the 1 wire port and return  0 no presence detect
 *                                           1 presence pulse no alarm
 *                                           2 alarm followed by presence
 *                                           3 short circuit to ground
 *                                           4 no com port found
 *
 * The global variable 'com_port' must be set to the com port that the
 * DS9097 COM Port Adapter is attached to before calling this routine.
 *
 */
uchar TouchReset( void )
{
     uint SPA,F,X,Y,tmp,trst=0;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* return if there is no address */
     if (!SPA) return 4;

     /* serial port initialization */
     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+1,0x00);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA+1,0x00);  /* no interrupts */
     outportb(SPA+4,0x03);  /* RTS and DTR on */

     /* Initialize the time limit */
     M = *sysclk + 1;

     /* loop to clear the buffers */
     do { tmp = inportb(SPA+5) & 0x60;  } while (tmp != 0x60);

     /* flush input */
     while (inportb(SPA+5) & 0x1) X = inportb(SPA);

     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA+1,0x00);  /* baud rate is 10473 */
     outportb(SPA  ,0x0B);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA  ,0xF0);  /* send the reset pulse */

     /* wait until character back or timeout */
     do
     {
          Y = inportb(SPA+5);
          F = Y & 0x1;
     } while ( !F && (*sysclk <= M) );


     if (F) X = inportb(SPA);
     else   return 3;

     if (X != 0xF0)           /* if more bits back than sent then there */
     {                        /* is a device  if framing error or break */
          trst = TR;
          if ( (Y & 0x18) != 0 )
          {
               trst = 2;

               /* loop to clear the buffers */
               do { tmp = inportb(SPA+5) & 0x60;  } while (tmp != 0x60);

               /* wait until character back or timeout */
               do
               {
                    Y = inportb(SPA+5);
                    F = Y & 0x1;
               } while ( !F && (*sysclk <= M) );

               if (F) X = inportb(SPA);
               else return 3;
          }
     }

     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */

     return trst;
}


/*------------------------------------------------------------------------
 * This is the 1-Wire routine 'TouchByte', sometimes called 'DataByte'.
 * It transmits 8 bits onto the 1-Wire data line and receives 8 bits
 * concurrently. The global variable 'com_port' must be set to the
 * com port that the serial brick is attached to before calling this
 * routine. This com port must also be set to 115200 baud, 8 dta, 1 stp,
 * and no parity. This routine returns the uchar 8 bit value received.
 * If it times out waiting for a character then 0xFF is returned.
 */
uchar TouchByte(uchar outch)
{
     uchar inch=0,sendbit,Mask=1;
     uint SPA;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* Initialize the time limit */
     M = *sysclk + 2;

     /* wait to TBE and TSRE */
     do {} while ( (inportb(SPA+5) & 0x60) != 0x60 );

     /* flush input */
     while ( (inportb(SPA+5) & 0x1) )
          inportb(SPA);

     /* get first bit ready to go out */
     sendbit = (outch & 0x1) ? 0xFF : 0x00;

     /* loop to send and receive 8 bits */
     do
     {
          outportb(SPA,sendbit);  /* send out the bit */

          /* get next bit ready to go out */
          Mask <<= 1;
          sendbit = (outch & Mask) ? 0xFF : 0x00;

          /* shift input char over ready for next bit */
          inch >>= 1;

          /* loop to look for the incoming bit */
          for (;;)
          {
               /* return if out of time */
               if ( *sysclk > M )
                  return 0xFF;

               if ( inportb(SPA+5) & 0x01 )
               {
                  inch |= ((inportb(SPA) & 0x01) ? 0x80 : 0x00);
                  break;
               }
          }

     } while (Mask);

     return inch;   /* return the input char */
}


/*------------------------------------------------------------------------
 * This is the 1-wire routine 'TouchBit'. It is used in the search rom
 * operation. It sends out one bit on the 1-wire and returns one bit.
 * The global variable 'com_port' must be set to the com port that the
 * DS9097 COM Port Adapter is attached to before calling this routine.
 * This com port must also be set to 115200 baud, 8 dta, 1 stp, and no
 * parity. If this routine times out waiting for the bit character
 * then 0x01 is returned.
 */
uchar TouchBit(uchar outbit)
{
     uchar sendbit;
     uint SPA;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* Initialize the time limit */
     M = *sysclk + 2;

     /* wait to TBE and TSRE */
     do {} while ( (inportb(SPA+5) & 0x60) != 0x60 );

     /* flush input */
     while ( (inportb(SPA+5) & 0x1) )
          inportb(SPA);

     /* get bit ready to go out */
     sendbit = (outbit & 0x1) ? 0xFF : 0x00;

     outportb(SPA,sendbit);  /* send out the bit */

     /* loop to look for the incoming bit */
     for (;;)
     {
          /* return if out of time */
          if ( *sysclk > M )
             return 1;

          if ( inportb(SPA+5) & 0x01 )
               return (inportb(SPA) & 0x01);  /* return the input bit */
     }
}

