/*---------Copyright (C) 1991,2 Dallas Semiconductor Corporation------------
 *
 *  SUBKEY_P - change the partition 0 password and id
 *
 *
 *  Date:      May 15, 1992
 *  Compiler:  Borland C++ 3.0
 *  Externals: TOUCHIOS.OBJ
 *  Version:   1.1 - cfs
 *
 *  Syntax:  subkey_p (com port #) (new_password) (new_id)
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>

#define PARTITION_0 0

typedef unsigned char uchar;
typedef unsigned int  uint;

/*
 * TOUCHIO routines
 */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar TouchByte( uchar );
extern uchar Next( void );
extern uchar First( void );
extern uchar Access( void );
extern uchar Access90( void );
extern uchar * RomData( void );
extern uint  crc16(int);
extern uchar dowcrc(uchar);

/* Global Variables need for TOUCHIOS.OBJ*/
int   com_port;
uint  CRC16;
uchar DOWCRC;


/* local functionn prototypes */
void main(int, char **);
void write_password_id(uchar *, uchar *);
int  cmpids(uchar *, uchar *);
void get_scratchpad(char *);
void set_scratchpad(char *);
void move_all_blocks(int, char *);
void get_secure_data(int, uchar *, uchar *, uchar *);
void set_password(int, uchar *, uchar *);


union                  /* this union equates two bytes to a 16 integer */
{
   unsigned int  lohiw;
   unsigned char lohi[2];
}  crcb;

/*
 * get command line arguments,
 * build the password and id fields
 * make sure we have a port with a brick
 */

void main(int argc, char **argv)
{
   int   i;
   int   n;
   uchar pwd[10];
   uchar id[10];

   if (argc != 4)
   {
      printf("usage: subkey_p com_port new_password new_id\n");
      printf("       - change the password and id in partition 0\n");
      printf("       - version 1.1");
      exit(1);
   }

   com_port = atoi(argv[1]);
   if (com_port < 1 || com_port > 4)
   {
      printf("error: com_port must be 1, 2, 3 or 4");
      exit(1);
   }

   /*
    *  check password and build the password field
    */
   if (strlen(argv[2]) > 8)
   {
      printf("error: the password must be 1 to 8 characters long");
      exit(1);
   }
   for (i = 0; i < 10; ++i)   /* put password into a zeroed field */
      pwd[i] = 0;
   strcpy((char*)(pwd), argv[2]);

   /*
    *  check id and build the id field
    */
   if ((n = strlen(argv[3])) > 5)
   {
      printf("error: the id must be 1 to 5 characters long");
      exit(1);
   }

   /*
    *  create an id field with <byte-count><ascii-id><CRC1616>
    */
   for (i = 0; i < 10; ++i)   /* put id into a zeroed field */
      id[i] = 0;
   id[0] = n;                 /* first byte is the length */
   CRC16   = 0;
   crc16(n);                  /* crc includes length byte */
   for (i = 0; i < n; ++i)    /* the next bytes are the id characters */
   {
      id[i+1] = argv[3][i];
      crc16(0xff & argv[3][i]);
   }
   crcb.lohiw = CRC16;
   id[n+1]    = crcb.lohi[0]; /* append low byte of crc and */
   id[n+2]    = crcb.lohi[1]; /* the high byte */

   /*
    *  make sure we can use this port
    */
   if (!Setup(com_port))
   {
      printf("error: invalid port");
      exit(2);
   }

   /*
    *  either no brick is connected, or com_port is to slow
    */
   if (TouchReset() == 3)
   {
      printf("error: serial port brick isn't connected");
      exit(2);
   }

   printf("\nTouch the wand to a touch memory or Press a key to abort.");

   write_password_id(pwd, id);
}

void write_password_id(uchar *newpwd, uchar *newid)
{
   int   i;
   uchar id[10];
   uchar data[50];


   for (i = 0; i < 10; ++i)
      id[i] = i;          /* jnk the id buffer we will read back */

   while(1)
   {
      if (kbhit())        /* user requested abort */
      {
         getch();
         printf("\nAborted");
         exit(1);
      }

      if (First())
      {
         /*
          * write out new password and new id
          */
         set_password(PARTITION_0, newpwd, newid);

         /*
          * we wrote the new pwd and new id if we can read back the same id.
          */
         get_secure_data(PARTITION_0, newpwd, id, data);
         if (cmpids(newid, id))
            break;
      }
   }
   printf("\nDone");
}

/*
 *  utility programs
 */

int cmpids(uchar *id0, uchar *id1)
{
   int i;

   for (i = 0; i < 8; ++i)
      if (id0[i] != id1[i])
         return 0;

   return 1;
}


/*
 * library routines.
 *    method is discussed in 1990-1991 data book, pages 565-578
 */

void get_scratchpad(char *buf)
   /* return 64 bytes of the scratch pad */
{
   int i;

   Access();
   TouchByte( 0x69);             /* get scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* read scratchpad, all 64 bytes */
      buf[i] = TouchByte(0xff);
}

void set_scratchpad(char *buf)
   /* set scratchpad with the 64 bytes pointed to by buf */
{
   int i;

   Access();
   TouchByte( 0x96);             /* write scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* get scratchpad, all 64 bytes */
      TouchByte(buf[i]);
}

char all_blks[8] = { 0x56, 0x56, 0x7f, 0x51, 0x57, 0x5d, 0x5a, 0x7f };

void move_all_blocks(int partition, char *pwd)
   /* move the data in the scratch pad to the partition requested.
      pwd must match the requested partitions password. */
{
   int i;

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x3c);              /* move scratchpad to partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* 'all blocks' command */
      TouchByte(all_blks[i]);

   for (i = 0; i < 8; ++i)      /* write password for comparison */
      TouchByte(pwd[i]);
}

void get_secure_data(int partition, uchar *pwd, uchar *id, uchar *data)
   /* return the current id and data for the requested partition
      with the given pwd. */
{
   int i;

   i = 0x10 + ((partition & 3) << 6);  /* address + partition */

   Access();
   TouchByte(0x66);              /* get secure data */
   TouchByte( i);                /* partition */
   TouchByte(~i);

   for (i = 0; i < 8; ++i)
      id[i] = TouchByte(0xff);   /* get current id for this partition */

   for (i = 0; i < 8; ++i)
      TouchByte(pwd[i]);         /* write password for comparison */

   for (i = 0; i < 48; ++i)
      data[i] = TouchByte(0xff); /* read secure data if passwords matched */
}

void set_password(int partition, uchar *newpwd, uchar *newid)
   /* write the newpwd and newid into the request partition */
{
   int i;
   int oldid[8];

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x5a);              /* set password for partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* read current id */
      oldid[i] = TouchByte(0xff);
   for (i = 0; i < 8; ++i)      /* write it back for comparison */
      TouchByte(oldid[i]);

   for (i = 0; i < 8; ++i)      /* write new id */
      TouchByte(newid[i]);
   for (i = 0; i < 8; ++i)      /* write new pwd */
      TouchByte(newpwd[i]);
}
