/*---------Copyright (C) 1991,2 Dallas Semiconductor Corporation------------
 *
 *  SUBKEY_R - reads and display romdata, scratchpad and partition 0
 *
 *
 *  Date:      May 15,1992
 *  Compiler:  Borland C++ 3.0
 *  Externals: TOUCHIOS.OBJ
 *  Version: 1.2 - cfs
 *
 *  Syntax:  subkey_r (com port #) (password)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <conio.h>

#define PARTITION_0 0

typedef unsigned char uchar;
typedef unsigned int  uint;

/*
 * TOUCHIO routines
 */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar TouchByte( uchar );
extern uchar Next( void );
extern uchar First( void );
extern uchar Access( void );
extern uchar Access90( void );
extern uchar * RomData( void );
extern uint  crc16(int);
extern uchar dowcrc(uchar);

/* Global Variables need for TOUCHIOS.OBJ*/
int   com_port;
uint  CRC16;
uchar DOWCRC;

/* Local function prototypes */
void read_data(uchar *);
int  validate_RomData(void);
int  validate_scratchpad(void);
int  validate_partition(uchar *);
int  chkcrc(uchar *, int);
int  cmpbuf(uchar *, uchar *);
void print_data(void);
void display_pwd_id_data(uchar *, uchar *, uchar *);
void keychar(uchar);
void get_scratchpad(uchar *);
void set_scratchpad(uchar *);
void move_all_blocks(int, uchar *);
void get_secure_data(int, uchar *, uchar *, uchar *);
void set_password(int, uchar *, uchar *);


/*
 * get command line arguments,
 * make sure we have a port with a brick
 */

void main(int argc, char **argv)
{
   int   i;
   uchar pwd[10];

   if (argc != 3)
   {
      printf("usage: subkey_r com_port password\n");
      printf("       - show rom-data, scratchpad-data and partition-0-data\n");
      printf("       - version 1.2");
      exit(1);
   }

   com_port = atoi(argv[1]);
   if (com_port < 1 || com_port > 4)
   {
      printf("error: com_port must be 1, 2, 3 or 4");
      exit(1);
   }

   if (strlen(argv[2]) > 8)
   {
      printf("error: the password must be 1 to 8 characters long");
      exit(1);
   }

   /*
    *  copy password into a null filled string
    */
   for (i = 0; i < 10; ++i)
      pwd[i] = 0;
   strcpy((char*)(pwd), argv[2]);

   /*
    *  make sure we can use this port
    */
   if (!Setup(com_port))
   {
      printf("error: invalid port");
      exit(2);
   }

   /*
    *  either no brick is connected, or com_port is to slow
    */
   if (TouchReset() == 3)
   {
      printf("error: serial port brick isn't connected");
      exit(2);
   }

   printf("\nTouch the wand to a touch memory or Press a key to abort.");

   read_data(pwd);
   print_data();
}

uchar *rdp;            /* romdata pointer */

int   sb = 0;          /* scratch pad buffer used last */
uchar sbuf[2][66];     /* scratch pad buffers */

int   pb = 0;          /* partition buffer used last */
uchar pbuf[2][66];     /* partition buffers */

int   vr   = 0;        /* valid rom - FALSE */
int   vs   = 0;        /* valid scratchpad - FALSE */
int   vp   = 0;        /* valid partition - FALSE */
int   done = 0;        /* all data read is valid */

void read_data(uchar *pwd)
{
   int i;

   rdp = RomData();    /* only need to do this once */

   for (i = 0; i < 64; ++i)  /* make sure buffers differ */
   {
      pbuf[0][i] = sbuf[0][i] = i;
      pbuf[1][i] = sbuf[1][i] = i + 10;
   }

   for (i = 0; i < 8; ++i)   /* make passwords equal */
      pbuf[0][i+8] = pbuf[1][i+8] = pwd[i];

   while(!done)        /* wait till all data is valid */
   {
      if (kbhit())     /* user requested abort */
      {
         getch();
         printf("\nAborted");
         exit(1);
      }

      if (First())
         if (!vr)
            validate_RomData();
         else if (!vs)
            validate_scratchpad();
         else if (!vp)
            validate_partition(pwd);
   }
}

int validate_RomData(void)          /* valid only if crc8 is correct */
{
   int i;
   int sum;

   sum = dowcrc(0);
   sum = dowcrc(sum);
   for (i = 0; i < 8; ++i)
      sum = dowcrc(rdp[i]);

   if (sum == 0)
      vr = 1;

   return vr;
}

int validate_scratchpad(void)       /* valid if id&data crc16s are okay, or */
{                           /*   if 2 consecutive reads are the same */
   sb = 1 - sb;             /* switch to the next sbuf */
   get_scratchpad(sbuf[sb]);

   if (chkcrc(&sbuf[sb][0], 5) && chkcrc(&sbuf[sb][16], 61))
      vs = 1;               /* crcs are okay */
   else if (cmpbuf(sbuf[0], sbuf[1]))
      vs = 1;               /* two consecutive reads are the same */

   return vs;
}

int validate_partition(uchar *pwd)/* valid if id&data crc16s are okay, or */
{                           /*   if 2 consecutive reads are the same */
   pb = 1 - pb;             /* switch to the next pbuf */
   get_secure_data(PARTITION_0, pwd, &pbuf[pb][0], &pbuf[pb][16]);

   if (chkcrc(&pbuf[pb][0], 5) && chkcrc(&pbuf[pb][16], 61))
      vp = done = 1;        /* crcs are okay */
   else if (cmpbuf(pbuf[0], pbuf[1]))
      vp = done = 1;        /* two consecutive reads are the same */

   return vp;
}

/*
 *  utility programs
 */

int chkcrc(uchar *buf, int max)
{
   int i;
   int n;

   n = buf[0];

   if (n > max)
      return 0;

   CRC16 = 0;
   n  += 3;
   for (i = 0; i < n; ++i)
      crc16(buf[i] & 0xff);

   if (CRC16)          /* a non zero value is bad */
      return 0;

   return 1;
}

int cmpbuf(uchar *buf0, uchar *buf1)
{
   int i;

   for (i = 0; i < 64; ++i)
      if (buf0[i] != buf1[i])
         return 0;

   return 1;
}

void print_data(void)
{
   int i;

   printf("\n    Romdata: ");
   for (i = 0; i < 8; ++i)
      printf("%2.2x ", rdp[i] & 0xff);

   printf("\n ScratchPad:");
   display_pwd_id_data(&sbuf[sb][8], &sbuf[sb][0], &sbuf[sb][16]);

   printf("\nPartition 0:");
   display_pwd_id_data(&pbuf[pb][8], &pbuf[pb][0], &pbuf[pb][16]);
}

void display_pwd_id_data(uchar *pwd, uchar *id, uchar *data)
{
   int i;
   int n;

   printf("\n   Password: ");   /* all 8 bytes are the password */
   for (i = 0; i < 8; ++i)
      keychar(pwd[i]);

   printf("\n         Id: ");   /* <byte count>(id)(CRC16lo)(CRC16hi) */
   if (chkcrc(id, 5))
   {
      n = id[0];
      for (i = 1; i <= n; ++i)
         keychar(id[i]);
   }
   else
   {
      for (i = 0; i < 8; ++i)
         keychar(id[i]);
      printf("\n             <BAD CRC IN ID FIELD>");
   }

   printf("\n       Data: ");   /* <byte count>(data)(CRC16lo)(CRC16hi) */
   if (chkcrc(data, 61))
   {
      n = data[0];
      for (i = 1; i <= n; ++i)
         keychar(data[i]);
   }
   else
   {
      for (i = 0; i < 64; ++i)
         keychar(data[i]);
      printf("\n             <BAD CRC IN DATA FIELD>");
   }
}

void keychar(uchar ch)
{
   if (isprint(ch))
      putchar(ch);
   else
      putchar('.');
}


/*
 * library routines.
 *    method is discussed in 1990-1991 data book, pages 565-578
 */

void get_scratchpad(uchar *buf)
   /* return 64 bytes of the scratch pad */
{
   int i;

   Access();
   TouchByte( 0x69);             /* get scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* read scratchpad, all 64 bytes */
      buf[i] = TouchByte(0xff);
}

void set_scratchpad(uchar *buf)
   /* set scratchpad with the 64 bytes pointed to by buf */
{
   int i;

   Access();
   TouchByte( 0x96);             /* write scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* get scratchpad, all 64 bytes */
      TouchByte(buf[i]);
}

char all_blks[8] = { 0x56, 0x56, 0x7f, 0x51, 0x57, 0x5d, 0x5a, 0x7f };

void move_all_blocks(int partition, uchar *pwd)
   /* move the data in the scratch pad to the partition requested.
      pwd must match the requested partitions password. */
{
   int i;

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x3c);              /* move scratchpad to partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* 'all blocks' command */
      TouchByte(all_blks[i]);

   for (i = 0; i < 8; ++i)      /* write password for comparison */
      TouchByte(pwd[i]);
}

void get_secure_data(int partition, uchar *pwd, uchar *id, uchar *data)
   /* return the current id and data for the requested partition
      with the given pwd. */
{
   int i;

   i = 0x10 + ((partition & 3) << 6);  /* address + partition */

   Access();
   TouchByte(0x66);              /* get secure data */
   TouchByte( i);                /* partition */
   TouchByte(~i);

   for (i = 0; i < 8; ++i)
      id[i] = TouchByte(0xff);   /* get current id for this partition */

   for (i = 0; i < 8; ++i)
      TouchByte(pwd[i]);         /* write password for comparison */

   for (i = 0; i < 48; ++i)
      data[i] = TouchByte(0xff); /* read secure data if passwords matched */
}

void set_password(int partition, uchar *newpwd, uchar *newid)
   /* write the newpwd and newid into the request partition */
{
   int i;
   int oldid[8];

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x5a);              /* set password for partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* read current id */
      oldid[i] = TouchByte(0xff);
   for (i = 0; i < 8; ++i)      /* write it back for comparison */
      TouchByte(oldid[i]);

   for (i = 0; i < 8; ++i)      /* write new id */
      TouchByte(newid[i]);
   for (i = 0; i < 8; ++i)      /* write new pwd */
      TouchByte(newpwd[i]);
}

