/*---------Copyright (C) 1991,2 Dallas Semiconductor Corporation------------
 *
 *  SUBKEY_W - change the partition 0 data field
 *
 *  Date:      May 15, 1992
 *  Compiler:  Borland C++ 3.0
 *  Externals: TOUHCIOS.OBJ
 *  Version: 1.1
 *
 *  Syntax: subkey_w (com port #) (current_password) (new_data)
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>


#define PARTITION_0 0

typedef unsigned char uchar;
typedef unsigned int  uint;

/*
 * TOUCHIO routines
 */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar TouchByte( uchar );
extern uchar Next( void );
extern uchar First( void );
extern uchar Access( void );
extern uchar Access90( void );
extern uchar * RomData( void );
extern uint  crc16(int);
extern uchar dowcrc(uchar);

/* Global Variables need for TOUCHIOS.OBJ*/
int   com_port;
uint  CRC16;
uchar DOWCRC;

/* local functionn prototypes */
void main(int, char **);
void write_data(uchar *, uchar *);
int  make_new_buffer(uchar *, uchar *, uchar *, uchar *);
int  get_and_validate_scratchpad(uchar *);
int  set_and_validate_scratchpad(uchar *);
int  validate_scratchpad(void);
int  get_and_validate_id(uchar *);
int  move_and_validate(uchar *, int *);
int  validate_partition(uchar*);
int  cmpbuf(uchar *, uchar *, int);
int  chkcrc(uchar *, int);
void get_scratchpad(uchar *);
void set_scratchpad(uchar *);
void move_all_blocks(int, uchar *);
void get_secure_data(int, uchar *, uchar *, uchar *);
void set_password(int, uchar *, uchar *);

union                  /* this union equates two bytes to a 16 integer */
{
   unsigned int  lohiw;
   unsigned char lohi[2];
}  crcb;

/*
 * get command line arguments,
 * build the password and data fields
 * make sure we have a port with a brick
 */

void main(int argc, char **argv)
{
   int   i;
   int   n;
   uchar pwd[10];
   uchar data[50];

   if (argc != 4)
   {
      printf("usage: subkey_w com_port current_password new_data\n");
      printf("       - change the data in partition 0\n");
      printf("       - version 1.1");
      exit(1);
   }

   com_port = atoi(argv[1]);
   if (com_port < 1 || com_port > 4)
   {
      printf("error: com_port must be 1, 2, 3 or 4");
      exit(1);
   }

   /*
    *  check password and build the password field
    */
   if (strlen(argv[2]) > 8)
   {
      printf("error: the password must be 1 to 8 characters long");
      exit(1);
   }
   for (i = 0; i < 10; ++i)   /* put password into a zeroed field */
      pwd[i] = 0;
   strcpy((char*)(pwd), argv[2]);

   /*
    *  check data and build the data field
    */
   if ((n = strlen(argv[3])) > 45)
   {
      printf("error: data must be 1 to 45 characters long");
      exit(1);
   }

   /*
    *  create a data field with <byte-count><ascii-id><crc16>
    */
   for (i = 0; i < 48; ++i)   /* put data into a zeroed field */
      data[i] = 0;
   data[0] = n;               /* first byte is the length */
   CRC16   = 0;
   crc16(n);                  /* crc includes length byte */
   for (i = 0; i < n; ++i)    /* the next bytes are the data characters */
   {
      data[i+1] = argv[3][i];
      crc16(0xff & argv[3][i]);
   }
   crcb.lohiw = CRC16;
   data[n+1]  = crcb.lohi[0]; /* append low byte of crc and */
   data[n+2]  = crcb.lohi[1]; /* the high byte */

   /*
    *  make sure we can use this port
    */
   if (!Setup(com_port))
   {
      printf("error: invalid port");
      exit(2);
   }

   /*
    *  either no brick is connected, or com_port is to slow
    */
   if (TouchReset() == 3)
   {
      printf("error: serial port brick isn't connected");
      exit(2);
   }

   printf("\nTouch the wand to a touch memory or Press a key to abort.");

   write_data(pwd, data);
}

void write_data(uchar *newpwd, uchar *newdata)
{
   int   good_pwd;
   int   seq;
   uchar currid[8];
   uchar orig[64];
   uchar new[64];

   seq      = 0;              /* start sequencing from zero */
   good_pwd = 0;              /* assume we are using bad password */

   while(1)
   {
      if (kbhit())            /* user requested abort */
      {
         getch();
         printf("\nAborted");
         exit(1);
      }

      if (First())            /* execute the following commands in seqence */
      {                       /* - each command returns one on success     */

         if (seq == 0)        /* get current scratch so it can be restored */
            seq += get_and_validate_scratchpad(orig);

         else if (seq == 1)   /* get the ID */
            seq += get_and_validate_id(currid);

         else if (seq == 2)   /* create a new buffer to write into partition */
            seq += make_new_buffer(new, newpwd, currid, newdata);

         else if (seq == 3)   /* move new buffer into scratchpad, check it */
            seq += set_and_validate_scratchpad(new);

         else if (seq == 4)   /* move scratchpad and check it destination */
            seq += move_and_validate(new, &good_pwd);

         else if (seq == 5)   /* restore the original scratch pad */
            seq += set_and_validate_scratchpad(orig);

         else if (seq == 6)   /* if we get here we are done */
            break;
      }
   }
   if (good_pwd)
      printf("\nDone");
   else
      printf("\nNothing changed.  Password is probably incorrect");
}

int make_new_buffer(uchar *new, uchar *newpwd, uchar *currid, uchar *newdata)
{
   int i;

   for (i = 0; i < 8; ++i)
   {
      new[i]   = currid[i];   /* to  0 thru  7 */
      new[i+8] = newpwd[i];   /* to  8 thru 15 */
   }
   for (i = 0; i < 48; ++i)
      new[i+16] = newdata[i]; /* to 16 thru 63 */

   return 1;                  /* always successful */
}

int   sb = 0;
uchar sbuf[2][64];

int get_and_validate_scratchpad(uchar *buf)
{
   int i;

   if (validate_scratchpad())
   {
      for (i = 0; i < 64; ++i)/* copy valid scratch buf into buf */
         buf[i] = sbuf[sb][i];
      return 1;               /* and we are successful */
   }
   else
      return 0;               /* not successful yet */
}

int set_and_validate_scratchpad(uchar *buf)
{
   set_scratchpad(buf);
   return (validate_scratchpad());
}

int validate_scratchpad(void) /* valid if id&data crc16s are okay, or */
{                             /*   if 2 consecutive reads are the same */
   sb = 1 - sb;               /* switch to the next sbuf */
   get_scratchpad(sbuf[sb]);

   if (chkcrc(&sbuf[sb][0], 5) && chkcrc(&sbuf[sb][16], 61))
      return 1;               /* crcs are okay */
   else if (cmpbuf(sbuf[0], sbuf[1], 64))
      return 1;               /* two consecutive reads are the same */

   return 0;                  /* still not valid */
}

int   pb = 0;
uchar pbuf[2][64];

int get_and_validate_id(uchar *id) /* valid if ID crc16 is okay, or */
{                             /*   if 2 consecutive reads are the same */
   int i;
   int status;

   status = 0;                /* assume failure */
   pb = 1 - pb;               /* switch to the next pbuf */
   get_secure_data(PARTITION_0, &pbuf[pb][8], &pbuf[pb][0], &pbuf[pb][16]);

   if (chkcrc(&pbuf[pb][0], 5))
      status = 1;             /* ID crc is okay */
   else if (cmpbuf(pbuf[0], pbuf[1], 8))
      status = 1;             /* two consecutive ID reads are the same */

   if (status)
      for (i = 0; i < 8; ++i)
         id[i] = pbuf[pb][i];

   return status;             /* no good id yet */
}

#define NATTEMPTS 3
int attempts = 0;             /* try NATTEMPTS before returning true */

int move_and_validate(uchar *buf, int *pwd_okay)
{
   uchar tmp[64];

   move_all_blocks(PARTITION_0, &buf[8]);

   get_secure_data(PARTITION_0, &buf[8], tmp, &tmp[16]);

   if (cmpbuf(&buf[16], &tmp[16], 48))
   {
      *pwd_okay = 1;          /* yep, password is okay */
      return 1;
   }

   if (++attempts >= NATTEMPTS)
   {
      *pwd_okay = 0;          /* password is probably not good */
      return 1;               /* but return success anyway     */
   }

   return 0;
}

int validate_partition(uchar *pwd)  /* valid if id&data crc16s are okay, or */
{                             /*   if 2 consecutive reads are the same */
   pb = 1 - pb;               /* switch to the next pbuf */
   get_secure_data(PARTITION_0, pwd, &pbuf[pb][0], &pbuf[pb][16]);

   if (chkcrc(&pbuf[pb][0], 5) && chkcrc(&pbuf[pb][16], 61))
      return 1;               /* crcs are okay */
   else if (cmpbuf(pbuf[0], pbuf[1], 64))
      return 1;               /* two consecutive reads are the same */

   return 0;
}

/*
 *  utility programs
 */

int cmpbuf(uchar *buf0, uchar *buf1, int length)
{
   int i;

   for (i = 0; i < length; ++i)
      if (buf0[i] != buf1[i])
         return 0;

   return 1;
}

int chkcrc(uchar *buf, int max)
{
   int i;
   int n;

   n = buf[0];

   if (n > max)
      return 0;

   CRC16 = 0;
   n  += 3;
   for (i = 0; i < n; ++i)
      crc16(buf[i] & 0xff);

   if (CRC16)                   /* a non zero value is bad */
      return 0;

   return 1;
}


/*
 * library routines.
 *    method is discussed in 1990-1991 data book, pages 565-578
 */

void get_scratchpad(uchar *buf)
   /* return 64 bytes of the scratch pad */
{
   int i;

   Access();
   TouchByte( 0x69);             /* get scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* read scratchpad, all 64 bytes */
      buf[i] = TouchByte(0xff);
}

void set_scratchpad(uchar *buf)
   /* set scratchpad with the 64 bytes pointed to by buf */
{
   int i;

   Access();
   TouchByte( 0x96);             /* write scratchpad */
   TouchByte( 0xc0);             /* from 0 */
   TouchByte(~0xc0);             /* complement of above */

   for (i = 0; i < 64; ++i)     /* get scratchpad, all 64 bytes */
      TouchByte(buf[i]);
}

char all_blks[8] = { 0x56, 0x56, 0x7f, 0x51, 0x57, 0x5d, 0x5a, 0x7f };

void move_all_blocks(int partition, uchar *pwd)
   /* move the data in the scratch pad to the partition requested.
      pwd must match the requested partitions password. */
{
   int i;

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x3c);              /* move scratchpad to partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* 'all blocks' command */
      TouchByte(all_blks[i]);

   for (i = 0; i < 8; ++i)      /* write password for comparison */
      TouchByte(pwd[i]);
}

void get_secure_data(int partition, uchar *pwd, uchar *id, uchar *data)
   /* return the current id and data for the requested partition
      with the given pwd. */
{
   int i;

   i = 0x10 + ((partition & 3) << 6);  /* address + partition */

   Access();
   TouchByte(0x66);              /* get secure data */
   TouchByte( i);                /* partition */
   TouchByte(~i);

   for (i = 0; i < 8; ++i)
      id[i] = TouchByte(0xff);   /* get current id for this partition */

   for (i = 0; i < 8; ++i)
      TouchByte(pwd[i]);         /* write password for comparison */

   for (i = 0; i < 48; ++i)
      data[i] = TouchByte(0xff); /* read secure data if passwords matched */
}

void set_password(int partition, uchar *newpwd, uchar *newid)
   /* write the newpwd and newid into the request partition */
{
   int i;
   int oldid[8];

   i = (partition & 3) << 6;    /* partition into high order bits */

   Access();
   TouchByte(0x5a);              /* set password for partition */
   TouchByte( i);                /* partition */
   TouchByte(~i);                /* complement of above */

   for (i = 0; i < 8; ++i)      /* read current id */
      oldid[i] = TouchByte(0xff);
   for (i = 0; i < 8; ++i)      /* write it back for comparison */
      TouchByte(oldid[i]);

   for (i = 0; i < 8; ++i)      /* write new id */
      TouchByte(newid[i]);
   for (i = 0; i < 8; ++i)      /* write new pwd */
      TouchByte(newpwd[i]);
}
