/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * RD_ROM.C - This program uses the Touch Memory routines in TOUCHIO.OBJ
 *            to read the rom code of a DOW device on the 1-Wire.
 *
 * Date: May 11, 1992
 * Compiler: Borland C++ 3.1
 * Externals: TOUCHIO.C
 * Version: 1.1
 *
 * Syntax:  RD_ROM [(com port #, default 1)]
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;

/* Global Variables */
const char TRUE   = 1;
const char FALSE  = 0;
/* Global Variables needed by TOUCHIO.C */
int   com_port;      /* serial com port for DOW */
uint  CRC16;         /* 16-bit cyclic redundancy check */
uchar DOWCRC;        /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* Function Prototypes for 1-wire routines in 'TOUCHIO.C'*/
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar First( void );
extern uchar * RomData( void );
extern uchar dowcrc(uchar);

/* Function Prototype for this module */
void main(int, char **);
int  Open_Port(int);
void correct_beep( void );
void Print_Rom( void );
int  Same_Rom( void );


/*--------------------------------------------------------------------------
 * This is the main routine in RD_ROM.C, it polls to read a Touch Memory
 * ROM ID and prints it. Any key quits the read.
 */
void main(int argc, char **argv)
{
     int   DeBounce=0;

     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                     RD_ROM - Version 1.1\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port number from the command line parameters */
     if (argc != 2) com_port = 1;
     else com_port = atoi(argv[1]);

     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     printf("\n\n           Waiting to read a Touch Memory ROM ID...\n"
            "                   (Press any KEY to quit)\n\n");

     /* loop to read and display rom codes */
     for (;;)
     {
          if (kbhit()) break;  /* quit the loop if key hit */

          /* look for a device on the 1-Wire */
          if (First())
          {
               if (!Same_Rom() || DeBounce > 20)
                  Print_Rom();
               DeBounce = 0;
          }
          else
             DeBounce++;

     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 * Check to see the current ROM CODE is the same as the last one.
 * It returns TRUE if the ROM CODE is the same and FALSE otherwise.
 */
int Same_Rom( void )
{
     static uchar OldRom[8];
     uchar  *ROM;
     int i,flag=1;

     ROM = RomData();    /* get the pointer to the current ROM CODE */

     /* loop to check to see if new rom is different from old */
     for(i = 0; i <= 7; i++)
     {
          if (ROM[i] != OldRom[i]) flag = FALSE;
          OldRom[i] = ROM[i];
     }

     return flag;
}


/*--------------------------------------------------------------------------
 *  This function prints the type and ROM CODE of the current device found
 *  on the 1-Wire.
 */
void Print_Rom( void )
{
     uchar *ROM;
     int i;

     correct_beep(); /* signal that a new ROM CODE was found */

     /* get the pointer to the ROM CODE */
     ROM = RomData();

     /* print the device type */
     switch(ROM[0])
     {
          case 1: i = 0; break;
          case 2: i = 1; break;
          case 8: i = 2; break;
          case 6: i = 3; break;
          case 4: i = 4; break;
     };
     printf("                             DOWCRCĿ   Serial NumberĿ  Family Code\n"
            "                                                        \n"
            "   DEVICE TYPE: DS199%d   ROM CODE: ",i);

     /* print the ROM CODE */
     for (i = 7; i >= 0; i--)
     {
         if (i == 6 || i == 0) printf(" ");
         printf(" %02X",ROM[i]);
     }
     printf("\n\n");
}


/*--------------------------------------------------------------------------
 *  This function emits a positive and pleasant beep.
 */
void correct_beep( void )
{
   sound(880);
   delay(50);
   sound(1760);
   delay(70);
   nosound();
}
