/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * ROM_SRCH.C - This program demonstrates using the multi-drop capabilities
 *              of the TOUCHIO.OBJ routines with DALLAS SEMICONDUCTOR
 *              1-Wire Touch Memory.
 *
 * Date: May 11, 1992
 * Compiler: Borland C++ 3.1
 * Externals: TOUCHIO.C
 * Version: 1.1
 *
 * Syntax:  ROM_SRCH [(com port #, default 1)]
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;

/* Global Variables */
const char TRUE   = 1;
const char FALSE  = 0;
/* Global Variables needed by TOUCHIO.C */
int   com_port;       /* serial com port for DOW */
uint  CRC16;          /* 16-bit cyclic redundancy check */
uchar DOWCRC;         /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* External Function Prototypes from 'TOUCHIO.C' */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar Next( void );
extern uchar First( void );
extern uchar * RomData( void );

/* Function Prototypes for ROM_SRCH.C */
void main(int, char **);
int  Open_Port( int );
void Print_Rom( void );


/*--------------------------------------------------------------------------
 * This is the main routine in ROM_SRCH.C.
 */
void main(int argc, char **argv)
{
     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                    ROM_SRCH - Version 1.1\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port number from the command line parameters */
     if (argc != 2) com_port = 1;
     else com_port = atoi(argv[1]);

     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     if (!First())
        printf("                   No Touch Memory on COM%d!\n"
               "    Touch Memory contact required during program execution.\n\n",com_port);
     else
     {
          printf("The following devices were found on the Dallas Semiconductor \n"
                 "1-Wire (DOW), on COM%d.\n\n"
                 "                             DOWCRCĿ   Serial NumberĿ  Family Code\n"
                 "                                                        \n"
                 ,com_port);

          /* loop to find and print all the rom codes on the DOW */
          do
          {
               /* Instead of just printing the ROM CODE of the device
                  found on the 1-wire, an Access() could be done at this
                  point.  After the Access(), any normal single device
                  operation can be done.  For example, a record could
                  be read from a page, or the time could be set.       */

               Print_Rom(); /* print the ROM CODE */

          } while (Next());
     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 *  This function prints the type and ROM CODE of the current device found
 *  on the 1-Wire.
 */
void Print_Rom( void )
{
     uchar *ROM;
     int i;

     /* get the pointer to the ROM CODE */
     ROM = RomData();

     /* print the device type */
     switch(ROM[0])
     {
          case 1: i = 0; break;
          case 2: i = 1; break;
          case 8: i = 2; break;
          case 6: i = 3; break;
          case 4: i = 4; break;
     };
     printf("   DEVICE TYPE: DS199%d   ROM CODE: ",i);

     /* print the ROM CODE */
     for (i = 7; i >= 0; i--)
     {
         if (i == 6 || i == 0) printf(" ");
         printf(" %02X",ROM[i]);
     }
     printf("\n");
}


