/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * WR_CLOCK.C - This program uses the Touch Memory routines in TOUCHIO.OBJ
 *              read the rom code of a DOW device on the 1-Wire.
 *
 * Date: May 11, 1992
 * Compiler: Borland C++ 3.1
 * Externals: TOUCHIO.C
 * Version: 1.1
 *
 * Syntax:  WR_CLOCK (com port #) (date XX/XX/XX) (time XX:XX:XX.XX)
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <string.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned long ulong;

/* Global Variables */
const char TRUE   =  1;
const char FALSE  =  0;
const char X      = -1;
/* Global Variables needed by TOUCHIO.C */
int   com_port;      /* serial com port for DOW */
uint  CRC16;         /* 16-bit cyclic redundancy check */
uchar DOWCRC;        /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* Function Prototypes for 1-wire routines in 'TOUCHIO.C'*/
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar First( void );
extern uchar TouchByte( uchar );
extern uchar Access( void );
extern uchar * RomData( void );
extern int   touchreset( void );
extern uchar dowcrc(uchar);

/* Function Prototype for this module */
void main(int, char **);
int  Open_Port(int);
void Format_TimeDate(char *, char *, uchar *);
int Write_TimeDate(uchar *);
int Read_TimeDate(uchar *);
int Read_Scratchpad(uchar *, uint *, uint *);
int Write_Scratchpad(uchar *, uint, uint);
int Copy_Scratchpad(uint, uint);
int Array_Comp(uchar *, uchar *, int);

/*--------------------------------------------------------------------------
 * This is the main routine in WR_CLOCK.C, it sets the real-time clock
 * of a DS1994 with the specified time and date.
 */
void main(int argc, char **argv)
{
     int   flag;
     uchar wr_str[10], *ROM;
     char *date_str, *time_str;

     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                    WR_CLOCK - Version 1.1\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port and time/date form command-line parameters*/
     if (argc != 4)
     {
           printf("SYNTAX: WR_CLOCK (com port #) (date XX/XX/XX) (time XX:XX:XX.XX)\n");
           exit(1);
     }
     else
     {
          com_port = atoi(argv[1]);
          date_str = argv[2];
          time_str = argv[3];
     }

     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     if (!First())
        printf("                   No Touch Memory on COM%d!\n"
               "    Touch Memory contact required during program execution.\n\n",com_port);
     else
     {
          ROM = RomData();  /* get the pointer to the ROM CODE */

          /* check to make sure part detected is a DS1994 */
          if (ROM[0] != 0x04)
          {
               printf("ERROR: This demo program requires a DS1994.\n");
               exit(1);
          }

          /* format the date/time arguments to 5 byte format */
          Format_TimeDate(date_str,time_str,wr_str);

          /* write the new date/time */
          flag = Write_TimeDate(wr_str);
          if (flag == TRUE)
               printf("Clock write successful.\n\n");
          else if (flag == FALSE-1)
               printf("Contact lost before clock write complete.\n\n");
          else
               printf("A KEY was pressed to abort the clock read.\n\n");
     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 * Format_TimeDate takes a time/date standard format string and converts it
 * into the 5 byte format used in DS1994's.
 */
int dm[] = { 0,0,31,59,90,120,151,181,212,243,273,304,334 };

void Format_TimeDate(char *date_str, char *time_str, uchar *wr_str)
{
     uint frsecond,second,minute,hour,day,month,year;
     ulong Sv,Bv,Xv;
     int tmp;

     /* extract the date/time values form the strings */
     tmp =  sscanf(date_str,"%d/%d/%d",&month,&day,&year);
     tmp += sscanf(time_str,"%d:%d:%d.%d",&hour,&minute,&second,&frsecond);

     printf("Input Date/Time interpreted as: %02d/%02d/%02d %02d:%02d:%02d.%02d\n\n",
             month,day,year,hour,minute,second,frsecond);

     /* check to see if all the scan fields were read correctly */
     if (tmp != 7)
     {
          printf("ERROR: The Date/Time arguments were not interpreted correctly.\n\n");
          exit(1);
     }

     /* convert the date/time values into the 5 byte format used in the touch */
     if (year < 70) Sv = year+32;
     else Sv = year - 68;

     if ((month > 2) && ( (Sv & 3) == 0))
        Bv = 1;
     else
        Bv = 0;

     Xv = 365 * (Sv-2) + (Sv-1)/4 + dm[month] + day + Bv - 1;

     Xv = 86400 * Xv + (ulong)(second) + 60*((ulong)(minute) + 60*(ulong)(hour));

     wr_str[0] = frsecond * 255 / 99;

     for (tmp = 1; tmp <= 4; tmp++)
     {
          wr_str[tmp] = Xv & 0xFF;
          Xv >>= 8;
     }
}



/*--------------------------------------------------------------------------
 * Write_TimeDate, writes 5 bytes to the 17th page of a DS1994, setting the
 * time/date register.
 */
int Write_TimeDate(uchar *tk)
{
     uchar rtk[40];
     int flag=1,i;

     if (!Access())          /* select the device that was found */
         return FALSE-1;

     /* write and copy the scratchpad with time/date string */
     flag = Write_Scratchpad(tk,0x202,6);
     if (flag == TRUE)
        Copy_Scratchpad(0x202,6);
     else
        return flag;

     /* verify the time/date by reading it back and comparing */
     flag = Read_TimeDate(rtk);

     if (flag == TRUE)
     {
          for (i = 2; i <= 6; i++)
          {
               if ( !((i == 2) || (i == 3)) ) /* ignore 2 lower lsbytes */
                    if (rtk[i] != tk[i-2])
                       return FALSE; /* compare with write string*/
          }
     }
     else
         return flag;

     return TRUE;
}


/*--------------------------------------------------------------------------
 * Read the bytes 1-6 of the 17th page of a DS1994. This contains the
 * control register and the current real-time clock value.
 */
int Read_TimeDate(uchar *tk)
{
     uint i,flag,first=1;
     uchar ch;
     int key=-1;
     tk[0] = 0;

     do
     {
          if (!Access())      /* select the device that was found */
             return FALSE-1;

          TouchByte(0xF0);         /* read memory command */

          TouchByte(0x01);         /* write the target address byte 1 */

          TouchByte(0x02);          /* write the target address byte 2 */

          flag = TRUE;            /* reset flag before loop */

          /* loop to read control register and time/date */
          for (i = 1; i <= 6; i++)
          {
               ch = TouchByte(0xFF);   /* read byte */

               if ( !(!first && (i == 2)) ) /* only read lsbyte once */
               {                            /* because fractional seconds */
                       if (ch != tk[i])     /* may change in multiple reads*/
                          flag = FALSE; /* compare with old set flag*/
                       tk[i] = ch;
               }
          }

          first = FALSE;

          if (kbhit()) key = getch();

     } while ((flag == FALSE) && (key == X));

     if (key != X) return key;

     /* check to see if Oscillator Enable is set */
     if (!(tk[1] & 0x10))
        printf("WARNING: The Oscillator Enable bit in the Control Register is\n"
               "         not set.  The clock will not increment!\n\n");

     if (Access()) return TRUE;
     else return FALSE-1;
}


/*--------------------------------------------------------------------------
 * Provides up to 32 bytes of the scratch pad of a selected 92,93,94
 * also sets the target address 'addr' and the offset. The Number
 * of bytes can be calculated from the 'addr' and 'offset'.
 */
int Read_Scratchpad(uchar *buf, uint *addr, uint *offset )
{
     int i,lpcnt=0,num_bytes,flag;
     uchar ch;

     /* loop until get the SCRATCHPAD free of error or 4 times max */
     do
     {
          flag = TRUE;                /* set the flag to true */

          if (!Access())         /* make sure still have a device */
             return FALSE-1;

          TouchByte(0xAA);             /* read scratchpad command */

          *addr = TouchByte(0xFF);     /* read the target address byte 1 */

                                      /* read the target address byte 2 */
          *addr = *addr | (TouchByte(0xFF) << 8);

          *offset = TouchByte(0xFF);   /* read the target address offset */

          /* calculate the number of bytes in scratchpad 1-32 */
          num_bytes = ((*addr & 0xE0) | (*offset & 0x1F)) - (*addr & 0xFF) + 1;

          for (i = 0; i < num_bytes; ++i)    /* read scratchpad contents */
          {
              ch = TouchByte(0xFF);   /* read byte */
              if (ch != buf[i]) flag = FALSE; /* compare with old set flag*/
              buf[i] = ch;
          }

     } while (lpcnt++ < 3 && flag == FALSE);

     if (flag == TRUE)
        return TRUE;
     else if (!Access())   /* verify that the fail was not because */
        return FALSE-1;    /* contact was lost */
     else
        return FALSE;
}


/*--------------------------------------------------------------------------
 * Write 32 bytes of the scratch pad of a selected 92,93,94 . Returns true
 * if access() returns true.
 */
int Write_Scratchpad(uchar *buf, uint addr ,uint offset)
{
   int   i;
   uint  taddr,toffset;
   uchar tbuf[33];

   if (!Access())       /* select the device that was found */
      return FALSE-1;

   TouchByte(0x0F);                /* write scratchpad command */

   TouchByte(addr & 0xFF);         /* write the target address byte 1 */

   TouchByte((addr & 0xFF00) >> 8);/* write the target address byte 2 */

                                   /* write num bytes scratchpad contents */
   for (i = 0; i <= (offset-(addr & 0xF)); i++)
      TouchByte(buf[i]);

   /* Check to make sure the data has been written correctly by reading */
   i = Read_Scratchpad(tbuf,&taddr,&toffset);
   if (i == TRUE)
      if (Array_Comp(buf,tbuf,i) &&       /* data same? */
          (addr == taddr) &&              /* target address same? */
          (offset == toffset) )           /* offset same? */
         return TRUE;
      else
         return FALSE;
   else
      return i;
}


/*--------------------------------------------------------------------------
 * Send the copy command to copy the scratchpad to NV ram.
 */
int Copy_Scratchpad(uint addr, uint offset)
{
     if (!Access())        /* select the device that was found */
        return FALSE-1;

     TouchByte(0x55);                 /* sent the copy command */

     TouchByte(addr & 0xFF);          /* write the target address byte 1 */

     TouchByte((addr & 0xFF00) >> 8); /* write the target address byte 2 */

     TouchByte(offset);               /* send the offset byte */

     return TRUE;
}


/*--------------------------------------------------------------------------
 * Compares two unsigned character arrays and returns TRUE
 * if they are the same
 */
int Array_Comp(uchar *ar1, uchar *ar2, int len)
{
     int i=0;

     if (len == -1)
        len = strlen((char*)(ar1));

     for (i = 0; i < len; i++)
     {
         if (ar1[i] != ar2[i])
            return FALSE;
     }

     return TRUE;
}
