Program CalTime;                    { Version 1.03   SMC   09/23/92 }
{
   This program attempts to calibrate the time in a DS1994 or DS1494
   from the DOS clock.

   Revisions:

      1.01 => 1.02 :  Added a second command line parameter, equal
                      to the number of hours to add to the DOS time.

      1.02 => 1.03 :  Modified TouchReset to accomodate alarms.
}

{$N+ Do Floating Point }
{$E+ But, Emulate It }

Uses DOS, CRT;                          { Required Pascal I/O units }

Const
   SPA : Word = 0;           { Currently active serial port address }

Function TouchReset(N: Byte): Boolean;
{
   This function transmits the one-wire protocol reset sequence to
   the device connected to COM port number N.  This sequence consists
   of a low pulse lasting a mimimum of 480 us followed by a high dead
   time lasting a mimimum of 480 us.  The function returns True if a
   presence detect pulse occurs during the dead time, otherwise
   it returns False.
}

Const
   Init : Array[1..4] of Boolean = (True, True, True, True);

Var
   S : Array[1..4] of Word Absolute $40:0;
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   F : Boolean;
   X, Y : Byte;

Begin
   SPA := 0; TouchReset := False;                  { Assume failure }
   If (N > 0) and (N < 5) and (S[N] > 0) then Begin  { Legal port # }
      SPA := S[N];                { Save active serial port address }
      If Init[N] then Begin   { Serial port requires initialization }
         Port[SPA + 3] := $83;                       { Set the DLAB }
         Port[SPA]     :=   1;                      { Bit rate is   }
         Port[SPA + 1] :=   0;                      {    115200 bps }
         Port[SPA + 3] :=   3;               { 8 dta, 1 stp, no par }
         Port[SPA + 1] :=   0;                      { No interrupts }
         Port[SPA + 4] :=   3;                     { RTS and DTR on }
         Init[N] := False;               { Initialization completed }
      End;
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do X := Port[SPA];     { Flush input }
      Port[SPA + 3] := $83;                              { Set DLAB }
      port[SPA+1] := 0;                        { Baud rate is 10473 }
      Port[SPA] := 11;
      Port[SPA + 3] := 3;               { 8 data, 1 stop, no parity }
      Port[SPA] := $F0;                      { Send the reset pulse }
      Repeat                 { Wait until character back or timeout }
         Y := Port[SPA + 5];
         F := Odd(Y);
      until F or (T > M);
      If F then X := Port[SPA] else X := $F0;
      If (X <> $F0) Then Begin        { If more bits back than sent }
         TouchReset := True;            {    then there is a device }
         If ((Y and $18) <> 0) Then Begin  { Framing error or break }
            Repeat until Port[SPA + 5] and $60 = $60;  { TBE & TSRE }
            Repeat F := Odd(Port[SPA + 5]) until F or (T > M);
            If F then X := Port[SPA];
         End;
      End;
      Port[SPA + 3] := $83;                          { Set the DLAB }
      Port[SPA]     :=   1;                { Bit rate is 115200 bps }
      Port[SPA + 3] :=   3;                  { 8 dta, 1 stp, no par }
   End;
End;

Function TouchByte(X: Byte): Byte;
{
   This function transmits the byte X to the device attached to the
   currently active serial port SPA, and returns a byte from the
   device as its value.
}

Var
   T    : LongInt Absolute $40:$6C;
   M    : LongInt;
   I, J : Byte;

Begin
   If SPA = 0 then TouchByte := X else Begin
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do I := Port[SPA];     { Flush input }
      I := 0; J := 0;      { Initialize output & input bit counters }
      Repeat
         If Odd(Port[SPA + 5]) then Begin
            Inc(J); If Odd(Port[SPA]) then X := X or $80;
         End else If (I<=J) and (Port[SPA+5] and $20 = $20) then Begin
            If Odd(X) then Port[SPA] := $FF else Port[SPA] := 0;
            X := X shr 1; Inc(I);
         End;
      Until (J = 8) or (T > M);
      While (J < 8) do Begin
         X := X shr 1 or $80;
         Inc(J)
      End;
      TouchByte := X;
   End;
End;

Procedure WByte(X: Byte);
{
   This procedure transmits the byte X to a Touch Memory.
}

Begin
   X := TouchByte(X);
End;

Function RByte: Byte;
{
   This function returns the byte received from a Touch Memory.
}

Begin
   RByte := TouchByte($FF);
End;

Const
   DM : Array[1..13] of Word =
      (0,31,59,90,120,151,181,212,243,273,304,334,365);

Procedure DateToDay(YR, MO, DA: LongInt; Var N: LongInt);
{
   Convert from Date to Day.
}

Begin
   N := 365 * (YR - 1970) + (YR - 1969) shr 2 + DM[MO]
      + DA + Byte((MO > 2) and (YR and 3 = 0)) - 1;
End;

Var
   S : Byte;

Function TimeIn(Var C: Comp): Boolean;
{
   This function attempts to read the real time clock register
   from a DS1494 attached to a serial port reader on COM port S.
   The function returns True if the read was successful, and False
   otherwise.  If the function returns True, then the value of the
   date/time in 256ths of a second is returned in the high precision
   variable C.
}

Type
   Arr = Array[0..7] of Byte;

Const
   SC : Comp = 0;

Var
   I : Byte;
   F : Boolean;

Begin
   F := TouchReset(S);              { Failure if no Presence signal }
   If F then Begin                         { Try to read the DS1494 }
      WByte($CC); WByte($F0); WByte(2); WByte(2);
      C := 0; For I := 0 to 4 do Arr(C)[I] := RByte;
      F := (C >= SC) and (C < SC + 64); SC := C;
   End;
   TimeIn := F;
End;

Function AlarmIn(Var A: Comp): Boolean;
{
   This function attempts to read the real time alarm register from a
   DS1494 attached to a serial port reader on COM port S.  The function
   returns True if the read was successful, and False otherwise.  If
   the function returns True, then the value in the real time alarm
   register is returned in the high precision variable A.  (Note that
   A is assumed to be a five byte unsigned integer.)
}

Type
   Arr = Array[0..7] of Byte;

Const
   SA : Comp = 0;

Var
   I : Byte;
   F : Boolean;

Begin
   F := TouchReset(S);              { Failure if no Presence signal }
   If F then Begin                    { Try to read alarm registers }
      WByte($CC); WByte($F0); WByte($10); WByte(2);
      A := 0; For I := 0 to 4 do Arr(A)[I] := RByte;
      F := (A = SA); SA := A;
   End;
   AlarmIn := F;
End;

Function AlarmOut(B: Comp): Boolean;
{
   This function attempts to write the value from the high precision
   variable B into the elapsed time alarm register of the DS1494
   attached to a serial port reader on COM port S.  The function
   returns True if the value in B was successfully written, and False
   otherwise.  (Note that B is assumed to be a five byte signed
   integer.)
}

Type
   Arr = Array[0..7] of Byte;

Var
   SB : Comp;
   I  : Byte;

Begin
   IF B < -5.49755813888E11 then B := -5.49755813888E11 else
   If B > 5.49755813887E11 then B := 5.49755813887E11;
   For I := 5 to 7 do Arr(B)[I] := 0;
   AlarmOut := False;                    { Assume unsuccessful write }
   If TouchReset(S) then Begin                { Found a Touch Memory }
      WByte($CC); WByte($F); WByte($15); WByte(2);
      SB := B; For I := 0 to 4 do WByte(Arr(SB)[I]);
      If TouchReset(S) then Begin
         WByte($CC); WByte($AA);
         For I := 1 to 3 do WByte($FF);
         SB := 0; For I := 0 to 4 do Arr(SB)[I] := RByte;
         If (B = SB) and TouchReset(S) then Begin
            WByte($CC); WByte($55); WByte($15); WByte(2); WByte($19);
            If TouchReset(S) then Begin
               WByte($CC); WByte($F0); WByte($15); WByte(2);
               SB := 0; For I := 0 to 4 do Arr(SB)[I] := RByte;
               AlarmOut := (SB = B);
            End;
         End;
      End;
   End;
End;

Var
   N              : Word;
   F              : Boolean;
   U, V, W        : Comp;
   L, M, Z        : LongInt;
   YR, MO, DA     : LongInt;
   NY, NM, ND     : Word;
   HR, MI, SE, HU : Word;

Begin
   TextColor(Yellow); TextBackground(Blue); ClrScr;
   S := 1;                                        { Default to COM1 }
   If ParamCount > 0 then Begin
      {$R+}
      Val(ParamStr(1), M, N);   { Get COM port number from cmd line }
      {$R-}
      If (N > 0) or (M < 1) or (M > 4) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else S := M;
   End;
   Z := 0;                          { Default to no time zone shift }
   If ParamCount > 1 then Begin
      {$R+}
      Val(ParamStr(2), M, N);   { Get COM port number from cmd line }
      {$R-}
      If (N > 0) or (M < -23) or (M > 23) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else Z := M;
   End;
   WriteLn('CALTIME   VER 1.03   09/23/92');
   WriteLn;
   WriteLn('WAITING TO CAL DATE AND TIME');
   WriteLn('PRESS <ESC> TO ABORT PROGRAM');
   WriteLn;
   GetDate(NY, NM, ND, N);
   YR := NY; MO := NM; DA := ND;
   DateToDay(YR, MO, DA, L);
   F := False;
   Repeat
      F := AlarmIn(W);
      If F then Begin
         GetTime(HR, MI, SE, HU);
         M := HR + Z; HU := 256 * HU div 100;
         M := (SE + 60 * (MI + 60 * M)) shl 8 + HU;
         V := 22118400; U := V * L + M;
         F := TimeIn(V);
         If F then Begin
            U := (V - W) / (U - V);
            If U > $7FFFFFFF then U := $7FFFFFFF;
            If U < -$80000000 then U := -$80000000;
            F := AlarmOut(U);
         End;
      End;
   Until F or KeyPressed and (ReadKey = #27);
   If F then WriteLn('DATE AND TIME SUCCESSFULLY CALIBRATED')
   else WriteLn('PROGRAM ABORTED - DATE/TIME NOT CALIBRATED');
End.
