{$N+}
Program ReadTime;               { Version 1.03   SMC, TP   09/23/92 }
{
   This program provides a demonstration of the procedures for
   reading, formatting, and displaying the calibration constants and
   the corrected and uncorrected date and time information from a
   DS1494 connected to one of the RS232C serial ports with a DS9097
   serial port adapter.  (The calibration constants are stored in
   the real time alarm register and the elapsed time alarm register.)

   Revisions:

   1.00 => 1.01:  Added comparison of corrected time with DOS clock.

   1.01 => 1.02:  Added time zone offset parameter for the DOS clock
                  on the command line.

   1.02 => 1.03:  Modified TouchReset to accomodate alarms.
}

{$N+ Do Floating Point }
{$E+ But, Emulate It }

Uses DOS, CRT;                          { Required Pascal I/O units }

Const
   SPA : Word = 0;           { Currently active serial port address }

Function TouchReset(N: Byte): Boolean;
{
   This function transmits the one-wire protocol reset sequence to
   the device connected to COM port number N.  This sequence consists
   of a low pulse lasting a mimimum of 480 us followed by a high dead
   time lasting a mimimum of 480 us.  The function returns True if a
   presence detect pulse occurs during the dead time, otherwise
   it returns False.
}

Const
   Init : Array[1..4] of Boolean = (True, True, True, True);

Var
   S : Array[1..4] of Word Absolute $40:0;
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   F : Boolean;
   X, Y : Byte;

Begin
   SPA := 0; TouchReset := False;                  { Assume failure }
   If (N > 0) and (N < 5) and (S[N] > 0) then Begin  { Legal port # }
      SPA := S[N];                { Save active serial port address }
      If Init[N] then Begin   { Serial port requires initialization }
         Port[SPA + 3] := $83;                       { Set the DLAB }
         Port[SPA]     :=   1;                      { Bit rate is   }
         Port[SPA + 1] :=   0;                      {    115200 bps }
         Port[SPA + 3] :=   3;               { 8 dta, 1 stp, no par }
         Port[SPA + 1] :=   0;                      { No interrupts }
         Port[SPA + 4] :=   3;                     { RTS and DTR on }
         Init[N] := False;               { Initialization completed }
      End;
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do X := Port[SPA];     { Flush input }
      Port[SPA + 3] := $83;                              { Set DLAB }
      port[SPA+1] := 0;                        { Baud rate is 10473 }
      Port[SPA] := 11;
      Port[SPA + 3] := 3;               { 8 data, 1 stop, no parity }
      Port[SPA] := $F0;                      { Send the reset pulse }
      Repeat                 { Wait until character back or timeout }
         Y := Port[SPA + 5];
         F := Odd(Y);
      until F or (T > M);
      If F then X := Port[SPA] else X := $F0;
      If (X <> $F0) Then Begin        { If more bits back than sent }
         TouchReset := True;            {    then there is a device }
         If ((Y and $18) <> 0) Then Begin  { Framing error or break }
            Repeat until Port[SPA + 5] and $60 = $60;  { TBE & TSRE }
            Repeat F := Odd(Port[SPA + 5]) until F or (T > M);
            If F then X := Port[SPA];
         End;
      End;
      Port[SPA + 3] := $83;                          { Set the DLAB }
      Port[SPA]     :=   1;                { Bit rate is 115200 bps }
      Port[SPA + 3] :=   3;                  { 8 dta, 1 stp, no par }
   End;
End;

Function TouchByte(X: Byte): Byte;
{
   This function transmits the byte X to the device attached to the
   currently active serial port SPA, and returns a byte from the
   device as its value.
}

Var
   T    : LongInt Absolute $40:$6C;
   M    : LongInt;
   I, J : Byte;

Begin
   If SPA = 0 then TouchByte := X else Begin
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do I := Port[SPA];     { Flush input }
      I := 0; J := 0;      { Initialize output & input bit counters }
      Repeat
         If Odd(Port[SPA + 5]) then Begin
            Inc(J); If Odd(Port[SPA]) then X := X or $80;
         End else If (I<=J) and (Port[SPA+5] and $20 = $20) then Begin
            If Odd(X) then Port[SPA] := $FF else Port[SPA] := 0;
            X := X shr 1; Inc(I);
         End;
      Until (J = 8) or (T > M);
      While (J < 8) do Begin
         X := X shr 1 or $80;
         Inc(J)
      End;
      TouchByte := X;
   End;
End;

Procedure WByte(X: Byte);
{
   This procedure transmits the byte X to a Touch Memory.
}

Begin
   X := TouchByte(X);
End;

Function RByte: Byte;
{
   This function returns the byte received from a Touch Memory.
}

Begin
   RByte := TouchByte($FF);
End;

Const
   DM : Array[1..13] of Word =
      (0,31,59,90,120,151,181,212,243,273,304,334,365);

Procedure DayToDate(I: LongInt; Var YR, MO, DA: LongInt);
{
   Convert from Day to Date.
}

Var
   J, N : LongInt;

Begin
   N := (I + 731) shl 2; YR := N div 1461;
   N := (N - 1461 * YR) shr 2;
   MO := 1 + (N + 1) div 30;
   J := DM[MO] + Byte((MO > 2) and (YR and 3 = 0));
   If N < J then Begin
      Dec(MO);
      J := DM[MO] + Byte((MO > 2) and (YR and 3 = 0));
   End;
   DA := N - J + 1; YR := YR + 1968;
End;

Procedure DateToDay(YR, MO, DA: LongInt; Var N: LongInt);
{
   Convert from Date to Day.
}

Begin
   N := 365 * (YR - 1970) + (YR - 1969) shr 2 + DM[MO]
      + DA + Byte((MO > 2) and (YR and 3 = 0)) - 1;
End;

Var
   S : Byte;

Function TimeIn(Var C: Comp): Boolean;
{
   This function attempts to read the real time clock register
   from a DS1494 attached to a serial port reader on COM port S.
   The function returns True if the read was successful, and False
   otherwise.  If the function returns True, then the value of the
   date/time in 256ths of a second is returned in the high precision
   variable C.
}

Type
   Arr = Array[0..7] of Byte;

Const
   SC : Comp = 0;

Var
   I : Byte;
   F : Boolean;

Begin
   F := TouchReset(S);              { Failure if no Presence signal }
   If F then Begin                         { Try to read the DS1494 }
      WByte($CC); WByte($F0); WByte(2); WByte(2);
      C := 0; For I := 0 to 4 do Arr(C)[I] := RByte;
      F := (C >= SC) and (C < SC + 64); SC := C;
   End;
   TimeIn := F;
End;

Function TimeOut(C: Comp): Boolean;
{
   This function attempts to write the date/time in 256ths of a
   second from the high precision variable C into real time clock
   register of the DS1494 attached to a serial port reader on COM
   port S.  The function returns True if the value in C was
   successfully written, and False otherwise.  The function
   also starts the clock running.
}

Type
   Arr = Array[0..7] of Byte;

Var
   SC : Comp;
   I  : Byte;

Begin
   TimeOut := False;                     { Assume unsuccessful write }
   If TouchReset(S) then Begin                { Found a Touch Memory }
      WByte($CC); WByte($F); WByte(0); WByte(2);
      WByte($38); WByte($50); SC := C;
      For I := 0 to 4 do WByte(Arr(SC)[I]);
      If TouchReset(S) then Begin
         WByte($CC); WByte($AA);
         For I := 1 to 3 do WByte($FF);
         If (RByte = $38) and (RByte = $50) then Begin
            SC := 0; For I := 0 to 4 do Arr(SC)[I] := RByte;
            If (C = SC) and TouchReset(S) then Begin
               WByte($CC); WByte($55); WByte(0); WByte(2); WByte(6);
               If TouchReset(S) then Begin
                  WByte($CC); WByte($F0); WByte(2); WByte(2);
                  SC := 0; For I := 0 to 4 do Arr(SC)[I] := RByte;
                  TimeOut := (SC >= C) and (SC < C + 64);
               End;
            End;
         End;
      End;
   End;
End;

Function AlarmIn(Var A, B: Comp): Boolean;
{
   This function attempts to read the real time alarm register and
   the elapsed time alarm register from a DS1494 attached to a serial
   port reader on COM port S.  The function returns True if the read
   was successful, and False otherwise.  If the function returns True,
   then the value in the real time alarm register is returned in the
   high precision variable A, and the value in the elapsed time alarm
   register is returned in the high precision variable B.  (Note that
   A is assumed to be a five byte unsigned integer, and B is assumed
   to be a five byte signed integer.)
}

Type
   Arr = Array[0..7] of Byte;

Const
   SA : Comp = 0;
   SB : Comp = 0;

Var
   I : Byte;
   F : Boolean;

Begin
   F := TouchReset(S);              { Failure if no Presence signal }
   If F then Begin                    { Try to read alarm registers }
      WByte($CC); WByte($F0); WByte($10); WByte(2);
      A := 0; For I := 0 to 4 do Arr(A)[I] := RByte;
      B := 0; For I := 0 to 4 do Arr(B)[I] := RByte;
      If Arr(B)[4] > $7F then For I := 5 to 7 do Arr(B)[I] := $FF;
      F := (A = SA) and (B = SB); SA := A; SB := B;
   End;
   AlarmIn := F;
End;

Function AlarmOut(A, B: Comp): Boolean;
{
   This function attempts to write the values from the high precision
   variables A and B into the real time alarm register and the elapsed
   time alarm register of the DS1494 attached to a serial port reader
   on COM port S.  The function returns True if the values in A and B
   were successfully written, and False otherwise.  (Note that A is
   assumed to be a five byte unsigned integer, and B is assumed to
   be a five byte signed integer.)
}

Type
   Arr = Array[0..7] of Byte;

Var
   SA, SB : Comp;
   I      : Byte;

Begin
   If A < 0 then A := 0 else
   If A > 1.099511627775E12 then A := 1.099511627775E12;
   IF B < -5.49755813888E11 then B := -5.49755813888E11 else
   If B > 5.49755813887E11 then B := 5.49755813887E11;
   For I := 5 to 7 do Arr(B)[I] := 0;
   AlarmOut := False;                    { Assume unsuccessful write }
   If TouchReset(S) then Begin                { Found a Touch Memory }
      WByte($CC); WByte($F); WByte($10); WByte(2);
      SA := A; For I := 0 to 4 do WByte(Arr(SA)[I]);
      SB := B; For I := 0 to 4 do WByte(Arr(SB)[I]);
      If TouchReset(S) then Begin
         WByte($CC); WByte($AA);
         For I := 1 to 3 do WByte($FF);
         SA := 0; For I := 0 to 4 do Arr(SA)[I] := RByte;
         SB := 0; For I := 0 to 4 do Arr(SB)[I] := RByte;
         If (A = SA) and (B = SB) and TouchReset(S) then Begin
            WByte($CC); WByte($55); WByte($10); WByte(2); WByte($19);
            If TouchReset(S) then Begin
               WByte($CC); WByte($F0); WByte($10); WByte(2);
               SA := 0; For I := 0 to 4 do Arr(SA)[I] := RByte;
               SB := 0; For I := 0 to 4 do Arr(SB)[I] := RByte;
               AlarmOut := (SA = A) and (SB = B);
            End;
         End;
      End;
   End;
End;

Function Pad(X: Byte): String;
{
   This function returns the two-character string equal to the numeric
   value of X, with zero fill on the left.  The value of X is assumed
   to lie between zero and 99.

}

Var
   S : String[2];

Begin
   Str(X, S);
   While Length(S) < 2 do S := '0' + S;
   Pad := S;
End;


function GetDOSTime : Comp;

var
  DOSNY,DOSNM,DOSND,DOSN     :Word;
  DOSHR,DOSMI,DOSSE,DOSHU    :Word;
  DOSYR,DOSMO,DOSDA,DOSL,DOSM:LongInt;
  DOSV,DOSU                  :Comp;

begin
  GetDate(DOSNY, DOSNM, DOSND, DOSN);
  DOSYR := DOSNY; DOSMO := DOSNM; DOSDA := DOSND;
  DateToDay(DOSYR, DOSMO, DOSDA, DOSL);
  GetTime(DOSHR, DOSMI, DOSSE, DOSHU);
  DOSM := DOSHR; DOSHU := 256 * DOSHU div 100;
  DOSM := (DOSSE + 60 * (DOSMI + 60 * DOSM)) shl 8 + DOSHU;
  DOSV := 22118400; DOSU := DOSV * DOSL + DOSM;
  GetDosTime := DOSU;
end;

Var
   N              : Word;
   A, B, U, V     : Comp;
   L, M           : LongInt;
   YR, MO, DA, Q  : LongInt;
   HR, MI, SE, HU : Byte;
   DV             : Comp;
   R              : Single;
   DOSTime        : Comp;

Procedure Display(V: Comp);
{
   This procedure formats and displays the value of V as a Date/Time.
}

Begin
   U := (V - 11059200) / 22118400; V := V - 22118400 * U;
   If V = 22118400 then Begin
      U := U + 1; V := 0
   End;
   Move(U, L, 4); Move(V, M, 4);
   DayToDate(L, YR, MO, DA);
   L := M div 256; HU := 100 * (M - 256 * L) div 256;
   M := L div 60; SE := L - 60 * M;
   L := M div 60; MI := M - 60 * L; HR := L;
   Write(Pad(MO),'/',Pad(DA),'/',YR,'  ');
   WriteLn(Pad(HR),':',Pad(MI),':',Pad(SE),'.',Pad(HU));
   WriteLn;
End;

Begin
   TextColor(Yellow); TextBackground(Blue); ClrScr;
   S := 1;                                        { Default to COM1 }
   If ParamCount > 0 then Begin
      {$R+}
      Val(ParamStr(1), M, N);   { Get COM port number from cmd line }
      {$R-}
      If (N > 0) or (M < 1) or (M > 4) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else S := M;
   End;
   Q := 0;                     { Default to no offset from DOS time }
   If ParamCount > 1 then Begin
      {$R+}
      Val(ParamStr(2), M, N);   { Get COM port number from cmd line }
      {$R-}
      If (N > 0) or (M < -23) or (M > 23) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else Q := M;
   End;
   WriteLn('READTIME   VER 1.03   09/23/92');
   Repeat
      If TimeIn(V) and AlarmIn(A, B) then Begin
         GotoXY(1, 3);
         Write('MEASURED DATE/TIME = '); ClrEol; Display(V);
         If (A > 0) and (B <> 0) then Begin
            DV := (V - A) / B; V := V + DV;
            Write('COMPUTED DATE/TIME = '); ClrEol; Display(V);
            Move(DV, L, 4); R := L / 256;
            Write('SECONDS DIFFERENCE = '); ClrEol; WriteLn(R:23:2);
            WriteLn;
            Write('CALIBRATION CONS A = '); ClrEol; Display(A);
            Write('CALIBRATION CONS B = '); ClrEol;
            If B < -$80000000 then B := -$80000000;
            If B > $7FFFFFFF then B := $7FFFFFFF;
            L := Round(B); WriteLn(L:23); WriteLn;
            DOSTime :=GetDOSTime;
            Write('DOS TIME           = '); ClrEol; Display (DOSTime);
            R := (V - DOSTime)/256 - 3600 * Q;
            Write('COMPUTED TIME - DOS TIME (SEC) = '); ClrEol;
            Writeln(R:11:2);
         End;
      End;
   Until KeyPressed and (ReadKey = #27);
End.
