PROGRAM EpromPC;                   { Version 0.13  HK  1-20-94 }
{
             COPYRIGHT (C) 1994 DALLAS SEMICONDUCTOR

   This program provides a demonstration of the procedures for
   reading and programming the memory contents of a DS1982 or DS2502
   1-wire EPROM part.

   Program execution is terminated by pressing the <ESC> key.

   Revisions by SMC:

      03/14/94 - EPROMPC8:  Changed version number to 0.08.
                            Standardized indentation of source code.
                            (Note that this source produces exactly
                            the same EXE file as the original code.)

      03/14/94 - EPROMPC9:  Changed version number to 0.09.
                            Revised program code to compile as a DOS
                            program instead of a Windows program.

      03/17/94 - EPROMPCA:  Changed version number to 0.10.
                            Made additional code changes to
                            standardize form without changing
                            functionality.

   Revisions by J.D.O.:

      04-15-94 - EPROMPCB:  Changed version number to 0.11.
                            Increased number of loops which can
                            be exited from the keyboard.
                            Added status copy to EPROM-EPROM copy.
                            Increased number of operations which
                            prompt for device. Added copyright notice.

      04-29-94 - EPROMPCC:  Changed version number to 0.12.
                            Removed QUIT (0) as default option.
                            Further increased number of loops which can
                            be exited from the keyboard. Reduced likelyhood
                            of non-EPROM device destruction due to application
                            of 11.5 V programming voltage.
                            Made numerous other (mostly cosmetic) changes per
                            "Software-Updates for EPROMPC" document
                            and subsequent meetings.

      05-06-94 - EPROMPCD:  Changed version number to 0.13.
                            Increased number of loops which can be exited
                            from the keyboard yet further.
}

{$B-}                              { Short-circuit boolean evaluation }
{$D-}                              { Debug information off }
{$F-}                              { Force far calls off }
{$I+}                              { I/O checking on }
{$L-}                              { Local symbol information off }
{$N-}                              { Numeric coprocessor not used }
{$R-}                              { Range checking off }
{$S-}                              { Stack checking off }
{$V-}                              { Strict variable-string checking off }
{$X+}                              { eXtended syntax allowed }

USES DOS, CRT;    { Required Pascal I/O units }

CONST
   SPA : WORD = 0;              { Currently active serial port address }

VAR
   RAM         : ARRAY[0..512] OF BYTE;
   ROM         : array [0..7] of byte;
   MyMission   : STRING;
   Dbyte       : ARRAY[0..127] OF BYTE;
   PGNM, BYCNT : BYTE;
   BYT         : ARRAY[0..127] OF BYTE;
   PROPW       : WORD;
   TA1         : WORD;
   TA2         : BYTE;
   TA1Sav      : BYTE;
   NumByt      : WORD;
   NumChar     : WORD;
   Pass,E1,E2  : BYTE;
   CmdTyp      : BYTE;
   Numtrys     : WORD;
   Hour        : WORD;
   Min1,Min2   : WORD;
   Sec1,Hun1   : WORD;
   Sec2,Hun2   : WORD;
   Diag        : BOOLEAN;
   B           : INTEGER;

PROCEDURE GoodBeep;

   BEGIN
      Sound(440); Delay(50); NoSound;
   END;

PROCEDURE BadBeep;

   BEGIN
      Sound(256); Delay(100); NoSound;
   END;

PROCEDURE CalcTime;

   BEGIN
      IF Min1 = Min2 THEN BEGIN
         IF Sec1 = Sec2 THEN BEGIN
            Writeln('Execution Time: ',Sec2-Sec1,'.',Hun2-Hun1,' seconds.');
         END ELSE BEGIN
            IF Hun2 < Hun1 THEN BEGIN
               Dec(Sec2);
               Hun2:= Hun2 + 100;
               Writeln('Execution Time: ',Sec2-Sec1,'.',Hun2-Hun1,
                    ' seconds.');
            END ELSE Writeln('Execution Time: ',Sec2-Sec1,'.',Hun2-Hun1,
                 ' seconds.');
         END;
      END ELSE BEGIN
         Sec2:= Sec2 + 60;
         IF Hun2 < Hun1 THEN BEGIN
            Dec(Sec2);
            Hun2:= Hun2 + 100;
            Writeln('Execution Time: ',Sec2-Sec1,'.',Hun2-Hun1,' seconds.');
         END ELSE Writeln('Execution Time: ',Sec2-Sec1,'.',Hun2-Hun1,
              ' seconds.');
      END;
   END;                         {end of procedure}

FUNCTION TouchReset(N: BYTE): BOOLEAN;
{
   This function transmits the one-wire protocol reset sequence to
   the device connected to COM port number N.  This sequence consists
   of a low pulse lasting a mimimum of 480 us followed by a high dead
   time lasting a mimimum of 480 us.  The function returns True if a
   presence detect pulse occurs during the dead time, otherwise
   it returns False.
}

   CONST
      Init : ARRAY[1..4] OF BOOLEAN = (TRUE, TRUE, TRUE, TRUE);

   VAR
      S    : ARRAY[1..4] OF WORD Absolute $40:0;
      T    : LongInt Absolute $40:$6C;
      M    : LongInt;
      F    : BOOLEAN;
      X, Y : BYTE;

   BEGIN
      SPA := 0;
      TouchReset := FALSE;                            { Assume failure }
      IF (N > 0) AND (N < 5) AND (S[N] > 0) THEN BEGIN  { Legal port # }
         SPA := S[N];                { Save active serial port address }
         IF Init[N] THEN BEGIN   { Serial port requires initialization }
            Port[SPA + 3] := $83;                       { Set the DLAB }
            Port[SPA]     :=   1;                      { Bit rate is   }
            Port[SPA + 1] :=   0;                      {    115200 bps }
            Port[SPA + 3] :=   3;               { 8 dta, 1 stp, no par }
            Port[SPA + 1] :=   0;                      { No interrupts }
            Port[SPA + 4] :=   3;                     { RTS and DTR on }
            Init[N] := FALSE;               { Initialization completed }
         END;
         M := T + 1;                       { Initialize the time limit }
         REPEAT UNTIL Port[SPA + 5] AND $60 = $60;  { Await TBE & TSRE }
         WHILE Odd(Port[SPA + 5]) DO X := Port[SPA];     { Flush input }
         Port[SPA + 3] := $83;                              { Set DLAB }
         Port[SPA+1] := 0;                        { Baud rate is 10473 }
         Port[SPA] := 11;
         Port[SPA + 3] := 3;               { 8 data, 1 stop, no parity }
         Port[SPA] := $F0;                      { Send the reset pulse }
         REPEAT                 { Wait until character back or timeout }
            Y := Port[SPA + 5];
            F := Odd(Y);
         UNTIL F OR (T > M);
         IF F THEN X := Port[SPA] ELSE X := $F0;
         IF (X <> $F0) THEN BEGIN        { If more bits back than sent }
            TouchReset := TRUE;          {      then there is a device }
            IF ((Y AND $18) <> 0) THEN BEGIN  { Framing error or break }
               REPEAT UNTIL Port[SPA + 5] AND $60 = $60;  { TBE & TSRE }
               REPEAT
                  F := Odd(Port[SPA + 5])
               UNTIL F OR (T > M);
               IF F THEN X := Port[SPA];
            END;
         END;
         Port[SPA + 3] := $83;                          { Set the DLAB }
         Port[SPA]     :=   1;                { Bit rate is 115200 bps }
         Port[SPA + 3] :=   3;                  { 8 dta, 1 stp, no par }
      END;
   END;

FUNCTION TouchByte(X: BYTE): BYTE;
{
   This function transmits the byte X to the device attached to the
   currently active serial port SPA, and returns a byte from the
   device as its value.
}

   VAR
      T    : LongInt Absolute $40:$6C;
      M    : LongInt;
      I, J : BYTE;

   BEGIN
      IF SPA = 0 THEN TouchByte := X ELSE BEGIN
         M := T + 1;                       { Initialize the time limit }
         REPEAT UNTIL Port[SPA + 5] AND $60 = $60;  { Await TBE & TSRE }
         WHILE Odd(Port[SPA + 5]) DO I := Port[SPA];     { Flush input }
         I := 0;
         J := 0;              { Initialize output & input bit counters }
         REPEAT
            IF Odd(Port[SPA + 5]) THEN BEGIN
               Inc(J);
               IF Odd(Port[SPA]) THEN X := X OR $80;
            END ELSE IF (I<=J) AND (Port[SPA+5] AND $20 = $20) THEN BEGIN
               IF Odd(X) THEN Port[SPA] := $FF ELSE Port[SPA] := 0;
               X := X ShR 1;
               Inc(I);
            END;
         UNTIL (J = 8) OR (T > M);
         WHILE (J < 8) DO BEGIN
            X := X ShR 1 OR $80;
            Inc(J);
         END;
         TouchByte := X;
      END;
   END;

FUNCTION TouchBit(X: BOOLEAN): BOOLEAN;
{
   This function transmits the bit corresponding to the boolean
   value of X to the device attached to the currently active serial
   port SPA, and returns the bit received from the device as its
   boolean value.
}

   VAR
      T : LongInt Absolute $40:$6C;
      M : LongInt;
      F : BOOLEAN;
      I : BYTE;

   BEGIN
      IF SPA = 0 THEN TouchBit := X ELSE BEGIN
         M := T + 1;                       { Initialize the time limit }
         REPEAT UNTIL Port[SPA + 5] AND $60 = $60;  { Await TBE & TSRE }
         WHILE Odd(Port[SPA + 5]) DO I := Port[SPA];     { Flush input }
         IF X THEN Port[SPA] := $FF ELSE Port[SPA] := 0;    { Send bit }
         REPEAT
            F := Odd(Port[SPA + 5])
         UNTIL F OR (T > M);                                 { Receive }
         TouchBit := NOT F OR Odd(Port[SPA]);    { Return received bit }
      END;
   END;

FUNCTION Hex(X: BYTE): STRING;
{
   This function returns the hexadecimal representation string
   of the byte passed to it.
}

   VAR
      S : STRING[2];
      I, J : BYTE;

   BEGIN
      S := '';
      FOR I := 1 TO 2 DO BEGIN
         J := X AND $F;
         X := X ShR 4;
         IF J > 9 THEN Inc(J, 7);
         S := CHAR(J + $30) + S;
      END;
      Hex := S;
   END;

VAR
   CRC8 : BYTE;

PROCEDURE DOWCRC(X: BYTE);
{
   This procedure calculates the cumulative Dallas Semiconductor
   one-wire CRC of all bytes passed to it.  The result accumulates
   in the global variable CRC8.
}

   CONST
      Table : ARRAY[0..255] OF BYTE = ( 0, 94,188,226, 97, 63,221,131,194,
           156,126, 32,163,253, 31, 65, 157,195, 33,127,252,162, 64, 30, 95,
           1,227,189, 62, 96,130,220, 35,125,159,193, 66, 28,254,160, 225,
           191, 93,  3,128,222, 60, 98, 190,224,  2, 92,223,129, 99, 61,124,
           34,192,158, 29, 67,161,255, 70, 24,250,164, 39,121,155, 197,132,
           218, 56,102,229,187, 89,  7, 219,133,103, 57,186,228, 6, 88, 25,
           71,165,251,120, 38,196,154, 101, 59,217,135,  4, 90, 184,230,167,
           249, 27, 69,198,152,122, 36, 248,166, 68, 26,153, 199, 37,123, 58,
           100,134,216, 91,  5,231,185, 140,210, 48,110, 237,179, 81, 15, 78,
           16,242,172, 47,113,147,205, 17, 79,173, 243,112, 46,204,146,211,
           141,111, 49,178,236, 14, 80, 175,241, 19, 77,206,144,114, 44,109,
           51,209,143, 12, 82,176,238, 50,108, 142,208, 83, 13,239,177,240,
           174, 76, 18,145,207, 45,115, 202, 148,118, 40,171,245, 23, 73,  8,
           86,180,234,105, 55,213,139, 87,  9,235,181, 54,104,138,212,149,
           203, 41,119,244,170, 72, 22, 233,183, 85, 11,136,214, 52,106, 43,
           117,151,201, 74, 20,246, 168, 116, 42,200,150, 21, 75,169,247,182,
           232, 10, 84,215,137, 107, 53);

   BEGIN
      CRC8 := Table[CRC8 XOR X];
   END;

VAR
   CRC : WORD;

PROCEDURE CRC16(X: BYTE);
{
   This procedure accumulates the CRC16 CRC of the data X in the
   global variable CRC.
}

   VAR
      I : BYTE;
      F : BOOLEAN;

   BEGIN
      FOR I := 1 TO 8 DO BEGIN
         F := Odd(X XOR CRC);
         CRC := CRC ShR 1;
         X := X ShR 1;
         IF F THEN CRC := CRC XOR $A001;
      END;
   END;

PROCEDURE WByte(X: BYTE);
{
   This procedure transmits the byte X to the Touch Memory.
}

   BEGIN
      X := TouchByte(X);
   END;

FUNCTION RByte: BYTE;
{
   This function returns the value of a byte received from the
   Touch Memory while also accumulating the CRC8 of the byte
   in the global variable CRC8.
}

   VAR
      X : BYTE;

   BEGIN
      X := TouchByte($FF);
      DOWCRC(X);
      RByte := X;
   END;

FUNCTION RByte16: BYTE;
{
   This function returns the value of a byte received from the
   Touch Memory while also accumulating the CRC16 of the byte
   in the global variable CRC.
}

   VAR
      X : BYTE;

   BEGIN
      X := TouchByte($FF);
      CRC16(X);
      RByte16:= X;
   END;


FUNCTION ReadNFile(S: BYTE; PNUM: BYTE): BOOLEAN;
{
   This function reads all pages of a file from EPROM
}

   VAR
      PL, TR : BYTE;
      I, AC  : WORD;
      F      : BOOLEAN;
      CRCSav : BYTE;

   BEGIN
      F := FALSE;
      AC := 1;
      REPEAT
         IF TouchReset(S) THEN BEGIN
            WByte($CC);
            WByte($F0);
            WByte(PNUM ShL 5);
            WByte(PNUM ShR 3);
            CRCSav :=TouchByte($FF);
            CRC8 := 0;
            DOWCRC($F0);
            DOWCRC(PNUM ShL 5);
            DOWCRC(PNUM ShR 3);
            IF CRCSav= CRC8 THEN BEGIN
               CRC := PNum;
               PL := RByte16;
               IF PL < 30 THEN BEGIN
                  FOR I := 1 TO PL - 1 DO BEGIN
                     Inc(AC);
                     RAM[AC] := RByte16;
                  END;
                  PNUM := RByte16;     { Read next page number }
                  TR := RByte16;
                  TR := RByte16;       { Read the CRC bytes }
                  IF CRC = $B001 THEN F := TRUE ELSE BEGIN
                     E1:=E1+1;
                     Numtrys:=Numtrys+1;
                     F:=FALSE;
                  END;          {CRC error on read}
               END ELSE BEGIN
                  F := FALSE;
                  Numtrys:=Numtrys+1;
               END;             {PL<30 error}
            END ELSE BEGIN
               E1:=E1+1;
               Numtrys:=Numtrys+1;
               F:=FALSE;
            END;                {CRC error on CMD,TA1,TA2}
         END ELSE F := FALSE;
      UNTIL (PNUM = 0) OR (NOT F) OR (Numtrys>20);
      IF F THEN BEGIN
         IF AC > $FF THEN BEGIN
            RAM[0] := $FF;
            RAM[1] := AC - $FF;
         END ELSE BEGIN
            RAM[0] := AC - 1;
            FOR I := 1 TO AC DO RAM[I] := RAM[I + 1];
         END;
      END;
      ReadNFile := F;
   END;

PROCEDURE ReadRAM(S: BYTE; VAR GoodRead: BOOLEAN);

   VAR
      I, N, AC    :  WORD;
      EX, M, PL   :  BYTE;
      NS          :  STRING;
      PNUM, TR    :  BYTE;
      Fill        :  WORD;
      Usepage     :  BYTE;
      ReadError   :  BOOLEAN;
      Dstruc      :  BOOLEAN;
      ExtMess     :  BOOLEAN;
      DefMess     :  BOOLEAN;

   BEGIN
      Write('Reading Touch Memory - ');
      GoodRead:=FALSE;
      Dstruc:=TRUE;
      ExtMess:=FALSE;
      DefMess:=FALSE;
      Numtrys:=1;
      E1:=0;
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      REPEAT
         IF TouchReset(S) THEN BEGIN
            ReadError:=FALSE;
            WByte($CC);
            WByte($F0);
            WByte(0);
            WByte(0);
            CRC := 0;
            M := 0;
            RAM[M] := RByte16;
            N := RAM[M];
            IF N = $FF THEN BEGIN
               Inc(M);
               RAM[M] := RByte16;
               N := N + RAM[M];
            END;
            Ex := RByte16;          { Read extension byte }
            IF Ex = $AA THEN BEGIN    { Found a directory }
               Inc(M);
               RAM[M]:=EX;
               IF (NOT ExtMess OR Dstruc) THEN BEGIN
                  Writeln('Found Extended File Structure.');
                  Dstruc:=FALSE;
                  ExtMess:=TRUE;
               END;
               IF N < 30 THEN BEGIN
                  FOR I := 2 TO N DO BEGIN   {Read directory page}
                     Inc(M);
                     RAM[M] := RByte16;
                  END;
                  Inc(M);
                  RAM[M]:=RByte16;
                  Inc(M);
                  RAM[M]:=Rbyte16;         {Read the CRC bytes }
                  IF CRC = $B001 THEN BEGIN
                     PNUM:=1;
                     Fill:=29-N;
                     IF Fill >0 THEN BEGIN
                        FOR I := N+3 TO 31 DO BEGIN
                        {Pad unused Bytes with $FF }
                           Inc(M);
                           RAM[M]:=$FF;
                           Rbyte16;
                        END;
                     END;
                     WHILE (PNUM<4) AND (Numtrys<20)
                                    AND NOT ReadError AND not KeyPressed
                          DO BEGIN
                        Usepage:=((RAM[4] ShR(PNUM))AND $01);
                        {read remaining pages, if page}
                        IF ((RAM[4] ShR(PNUM))AND $01)=$01 THEN BEGIN
                        {appears in used pages list, read}
                           CRC:= PNUM;
                           N:=Rbyte16;
                           Inc(M);
                           RAM[M]:=N;
                           {data, otherwise pad with $FF}
                           IF N < 30 THEN BEGIN
                              FOR I := 1 TO N DO BEGIN
                                 Inc(M);
                                 RAM[M] := RByte16;
                              END;
                              Inc(M);
                              RAM[M]:=RByte16;     {Read the CRC bytes }
                              Inc(M);
                              RAM[M]:=Rbyte16;
                              IF CRC = $B001 THEN BEGIN
                                 PNUM:=PNUM+1;
                                 IF PNUM=4 THEN BEGIN
                                    IF Diag THEN BEGIN
                                       GetTime(Hour,Min2,Sec2,Hun2);
                                       Writeln('Reading completed in ',
                                            Numtrys,' attempt(s).');
                                       CalcTime;
                                    END;
                                    GoodRead:=TRUE;
                                    Goodbeep;
                                 END;
                                 Fill:=29-N;
                                 IF Fill >0 THEN BEGIN
                                    FOR I := N+3 TO 31 DO BEGIN
                                    {Pad unused Bytes with $FF }
                                       Inc(M);
                                       RAM[M]:=$FF;
                                       Rbyte16;
                                    END;
                                 END;
                              END ELSE BEGIN
                                 E1:=E1+1;
                                 Numtrys:=Numtrys+1;
                                 ReadError:= TRUE;
                              END;     {CRC error pages 1,2,or3}
                           END ELSE BEGIN
                              Numtrys:=Numtrys+1;
                              ReadError:= TRUE;
                           END;  {N>30 error pages 1,2,or3}
                        END ELSE BEGIN
                           FOR I:=0 TO 31 DO BEGIN
                              Inc(M);
                              RAM[M]:=$FF;
                              Rbyte16;
                           END;
                           PNUM:=PNUM+1;
                           IF PNUM=4 THEN BEGIN
                              IF Diag THEN BEGIN
                                 GetTime(Hour,Min2,Sec2,Hun2);
                                 Writeln('Reading completed in ', Numtrys,
                                      ' attempt(s).');
                                 CalcTime;
                              END;
                              GoodRead:=TRUE;
                              Goodbeep;
                           END;
                        END;
                     END;
                  END ELSE BEGIN
                     E1:=E1+1;
                     Numtrys:=Numtrys+1;
                  END;          {CRC read error page 0}
               END ELSE Numtrys:=Numtrys+1;   {N>30 error page 0}
            END ELSE IF Ex < $80 THEN BEGIN  {found default data structure}
               IF NOT (DefMess AND Dstruc) THEN BEGIN
                  Writeln('Found Default Data Structure.');
                  Dstruc:=TRUE;
                  DefMess:=TRUE;
               END;
               Inc(M);
               RAM[M] := EX;
               Inc(M);
               IF N > 0 THEN FOR I := M TO N+2 DO RAM[I] := RByte16;
               IF CRC = $B001 THEN BEGIN
                  IF Diag THEN BEGIN
                     GetTime(Hour,Min2,Sec2,Hun2);
                     Writeln('Reading completed in ',Numtrys, ' attempt(s).');
                     CalcTime;
                  END;
                  GoodRead:=TRUE;
                  GoodBeep;
                  Fill:=125-N;
                  {Pad unused Bytes with $FF }
                  IF Fill>0 THEN FOR I := N+3 TO 127 DO RAM[I]:=$FF;
               END ELSE BEGIN
                  E1:=E1+1;
                  Numtrys:=Numtrys+1;
               END;
            END ELSE Numtrys:=Numtrys+1;  {Did not find default or extended}
         END;                             {data structure}
      UNTIL (GoodRead) OR (Numtrys>20) or KeyPressed;

      IF Numtrys>20 THEN BEGIN
         GetTime(Hour,Min2,Sec2,Hun2);
         BadBeep;
         Writeln('Unable to read after ',Numtrys-1,' attempt(s).');
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Errors Encountered during Reading.');
            CalcTime;
         END;
         Writeln;
      END;
   END;

PROCEDURE ReadEPROM(S: BYTE; VAR GoodRead: BOOLEAN);

   VAR
      I, N, AC,CTR:  WORD;
      EX, M, PL   :  BYTE;
      NS,Str      :  STRING;
      PNUM, TR    :  BYTE;
      Fill        :  WORD;
      Usepage     :  BYTE;
      CRCSav      :  BYTE;
      L           :  LongInt;
      NoFile      :  BOOLEAN;
      Fnam,Fext   :  STRING;
      Dstruc      :  BOOLEAN;
      ExtMess     :  BOOLEAN;
      DefMess     :  BOOLEAN;

   BEGIN
      GoodRead:=FALSE;
      NoFile:=FALSE;
      Numtrys:=1;
      Dstruc:=TRUE;
      ExtMess:=FALSE;
      DefMess:=FALSE;
      E1:=0;
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      REPEAT
         IF TouchReset(S) THEN BEGIN
            WByte($CC);
            WByte($F0);
            WByte(0);
            WByte(0);
            CRCSav:= TouchByte($FF);
            CRC8 := 0;
            DOWCRC($F0);
            DOWCRC(0);
            DOWCRC(0);
            IF CRCSav=CRC8 THEN BEGIN
               CRC := 0;
               M := 0;
               RAM[M] := RByte16;
               N := RAM[M];
               IF N = $FF THEN BEGIN
                  Inc(M);
                  RAM[M] := RByte16;
                  N := N + RAM[M];
               END;
               Ex := RByte16;   { Read extension byte }
               IF Ex = $AA THEN BEGIN   { Found a directory }
                  IF Dstruc OR NOT ExtMess THEN BEGIN
                     Writeln('Found Extended File Structure.');
                     Dstruc:=FALSE;
                     ExtMess:=TRUE;
                  END;
                  IF ReadNFile(S, 0) THEN BEGIN  { Read root directory }
                     N := RAM[0];
                     Ctr := 8;
                     IF N > 9 THEN BEGIN
                        Write('Files Found: ');
                        REPEAT
                           NS := '';
                           FOR I := 0 TO 4 DO NS := NS + CHAR(RAM[I + Ctr] );
                           FOR I := 1 TO 4 DO Write(NS[I]);
                           Write('.',(RAM[I+Ctr]),'      ');
                           Ctr := Ctr + 7;
                        UNTIL (Ctr > N);
                        IF N<16 THEN BEGIN
                           MyMission:=NS;
                           Writeln;
                        END ELSE BEGIN
                           Writeln;
                           Write('File to read (');
                           FOR I := 1 TO 4 DO Write(MyMission[I]);
                           Write('.');
                           Write(Ord(MyMission[5]));
                           Write('): ');
                           Readln(Str);
                           IF Length(Str)<>0 THEN BEGIN
                              for I := 1 to length(str) do begin
                                 Str[I] := upcase(Str[I]);
                              end;
                              Fnam:=Str;
                              Delete(Fnam,5,4); {put file name into Fnam}
                              Fext:= Str;
                              Delete(Fext,1,5); {put file extension into Fext}
                              {$R+} Val(Fext, L, N); {$R-}
                              MyMission:=Fnam + CHAR(l);
                           END;
                        END;
                        N := RAM[0];
                        Ctr := 8;
                        REPEAT
                           NS := '';
                           FOR I := 0 TO 4 DO NS := NS + CHAR(RAM[I + Ctr] );
                           Ctr := Ctr + 7;
                        UNTIL (NS=MyMission) OR (Ctr > N);
                        IF NS = MyMission THEN BEGIN
                           PNUM:=RAM[Ctr-2];
                           REPEAT
                              IF TouchReset(S) THEN BEGIN
                                 IF ReadNFile(S, PNUM) THEN BEGIN
                                    GoodRead:= TRUE;
                                    Goodbeep;
                                    FOR I:=1 TO RAM[0] DO BEGIN
                                       Write(CHAR(RAM[I]));
                                       IF RAM[TA1]=13 THEN Write(#10);
                                    END;
                                 END ELSE Numtrys:=Numtrys+1;
                              END;
                           UNTIL (GoodRead) OR (Numtrys>20);
                        END ELSE BEGIN
                           Writeln('File Not Found.');
                           NoFile:=TRUE;
                           Badbeep;
                        END;
                     END ELSE BEGIN
                        Writeln('No files in directory.');
                        NoFile:=TRUE;
                        Badbeep;
                     END;
                  END;
               END ELSE IF Ex < $80 THEN BEGIN {found default data structure}
                  IF NOT (DefMess AND Dstruc) THEN BEGIN
                     Writeln('Found Default Data Structure.');
                     Dstruc:=TRUE;
                     DefMess:=TRUE;
                  END;
                  Inc(M);
                  RAM[M] := EX;
                  Inc(M);
                  IF N > 0 THEN
                     FOR I := M TO N+2 DO RAM[I] := RByte16;
                  IF CRC = $B001 THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                     FOR TA1:=1 TO RAM[0] DO BEGIN
                        Write(Chr(RAM[TA1]));
                        IF RAM[TA1]=13 THEN Write(#10);
                     END;
                     Writeln;
                     GoodRead:=TRUE;
                     GoodBeep;
                     IF Diag THEN BEGIN
                        Writeln('Reading completed in ',Numtrys,
                             ' attempt(s).');
                        CalcTime;
                     END;
                  END ELSE BEGIN
                     E1:=E1+1;
                     Numtrys:=Numtrys+1;
                  END;
               END ELSE Numtrys:=Numtrys+1; {Did not find default or extended}
            END ELSE BEGIN
               E1:=E1+1;
               Numtrys:=Numtrys+1;
            END;                {CRC error on command, TA1, TA2}
         END;                   {data structure}
      UNTIL GoodRead OR (Numtrys>20) OR NoFile or KeyPressed;

      IF Numtrys>20 THEN BEGIN
         GetTime(Hour,Min2,Sec2,Hun2);
         BadBeep;
         Writeln('Not formatted as text. Use function (2) or (3) to display data in Hex format.');
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Errors Encountered during Reading.');
            CalcTime;
         END;
         Writeln;
      END;
   END;                         {end of ReadEPROM procedure}


PROCEDURE ReadEPROMcrc(K: BYTE; Output: BOOLEAN; VAR ReadSuccess: BOOLEAN);
{
   Read EPROM w/ Page CRC calculated
}

   VAR
      CRCSav      :  ARRAY[0..4] OF BYTE;
      Endloop     :  BOOLEAN;
      ReadError   :  BOOLEAN;
      X,J,I       :  BYTE;


   BEGIN
      WHILE (NumTrys<21) AND NOT ReadSuccess DO BEGIN
         IF TouchReset(K) THEN BEGIN
            TA1:= PGNM*32;
            ReadError:=FALSE;
            WByte($CC);
            Wbyte($C3);
            WByte(TA1);
            WByte(0);
            CRCSav[4] := TouchByte($FF);
            IF PGNM = 0 THEN X := CRCSav[4];
            CRC8 := 0;
            DOWCRC($C3);
            DOWCRC(TA1);
            DOWCRC(0);
            IF CRC8 = CRCSav[4] THEN BEGIN
               WHILE (PGNM < 4) AND NOT ReadError DO BEGIN
                  CRC8 := 0;
                  FOR BYCNT := 0 TO 31 DO BEGIN
                     BYT[TA1] := RByte;
                     Inc(TA1);
                  END;
                  CRCSav[PGNM] := TouchByte($FF);
                  IF CRCSav[PGNM] = CRC8 THEN PGNM:= PGNM+1 ELSE BEGIN
                     E1:=E1+1;
                     Numtrys:=Numtrys+1;
                     ReadError:=TRUE;
                  END;
               END;
               IF PGNM=4 THEN BEGIN
                  IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                  ReadSuccess := TRUE;
                  GoodBeep;
                  IF Output THEN BEGIN
                     Writeln;
                     Writeln('Initial CRC = ',Hex(X));
                     writeln('     0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F');
                     FOR PGNM:=0 TO 3 DO BEGIN
                        TA1:=PGNM*32;
                        J:=0;
                        FOR J:=0 TO 1 DO BEGIN
                           write(Hex(TA1), ' ');
                           FOR BYCNT := 0 TO 15 DO BEGIN
                              Write(Hex(BYT[TA1]):3);
                              Inc(TA1);
                           END;
                           writeln;
                        END;
                        Writeln('CRC for Page ',PGNM,': ',Hex(CRCSav[ PGNM]));
                     END;
                  END;
                  IF Diag THEN BEGIN
                     Writeln('Reading completed in ',Numtrys, ' attempt(s).');
                     CalcTime;
                  END;
               END;
            END ELSE BEGIN
               E1:=E1+1;
               Numtrys:=Numtrys+1;
            END;
         END;
      END;
   END;                         {end of procedure ReadEPROMcrc}


FUNCTION Pad(X: BYTE): STRING;

   VAR
      S : STRING[2];

   BEGIN
      Str(X, S);
      WHILE Length(S) < 2 DO S := '0' + S;
      Pad := S;
   END;


PROCEDURE TextIn;

   VAR
      X, Y         : ARRAY[0..508] OF WORD;
      M, SX, SY,N  : WORD;
      X1, Y1       : WORD;
      C            : CHAR;
      I,NumLine    : BYTE;
      X2, Y2       : WORD;

   BEGIN
      clrscr;
      X1:= WhereX;
      Y1:= WhereY;
      Writeln('Enter text below, exit by typing ~.');
      X2 := wherex;    Y2 := wherey;
      Writeln;
      N := 1;
      C := ReadKey;
      NumLine := 1;
      X[N] := WhereX;
      Y[N] := WhereY;
      WHILE (C <> '~') AND (N<126) DO BEGIN
         IF C = #8 THEN BEGIN
            IF N>1 THEN BEGIN
               Dec(N);
               GotoXY(X[N], Y[N]);
               IF Dbyte[N] > $20 THEN BEGIN
                  Write(' ');
                  GotoXY(X[N], Y[N]);
               END;
            END;
         END ELSE BEGIN
            Dbyte[N] := BYTE(C);
            Inc(N);
            Write(C);
            IF C = #13 THEN BEGIN
               Write(#10);
               NumLine := NumLine +1;
            END;
            X[N] := WhereX;
            Y[N] := WhereY;
         END;
         SX := WhereX;
         SY := WhereY;
         GotoXY(X2, Y2);
         ClrEol;
         Write('Number of possible characters remaining:',126 - N);
         GotoXY(SX, SY);
         C := ReadKey;
      END;
      Dec(N);
      Dbyte[0]:=N;
      CRC:=0;
      FOR I:=0 TO N DO BEGIN
         CRC16(Dbyte[I]);
      END;
      IF N>0 THEN BEGIN
         Dbyte[N+1]:=NOT Lo(CRC);
         Dbyte[N+2]:=NOT Hi(CRC);
      END ELSE BEGIN
         Dbyte[N+1]:=$FF;
         Dbyte[N+2]:=$FF;
      END;
      Writeln;
   END;


function IsEPROM(K: byte): boolean;  { determine if device can handle 12VDC }
var I: byte;
begin
    IsEPROM := false;
    if TouchReset(K) then begin
        WByte($33);
        CRC8 := 0;
        for I := 0 to 7 do begin
            ROM[I] := RByte;
        end;
        IsEPROM := ((ROM[0] and $7F) = 9) and (CRC8 = 0);
    end;
end;

PROCEDURE PulWidth(X: WORD);
{
   This procedure creates a fixed pulse width for programming that is
   independent of system clock speed. X is in units that are approximately
   microseconds for values greater than 400.
}

   VAR
      M, N : WORD;

   BEGIN
      InLine($FA);                                   {Turn off interrupts}
      Port[SPA + 4] := Port[SPA + 4] AND $FD; {Apply Program Pulse on RTS}
      Port[$43] := 0;                              {Freeze value in timer}
      M := Port[$40] ShL 8 OR Port[$40];             {Read value in timer}
      REPEAT                                   {Loop to consume real time}
         Port[$43] := 0;                     {Freeze value in timer again}
         N := Port[$40] ShL 8 OR Port[$40];      {Read new value in timer}
      UNTIL M - N >= X;                     {See if "X" usec have elapsed}
      Port[SPA + 4] := Port[SPA + 4] OR 2;        {Remove Program Voltage}
      InLine($FB);                                    {Turn interrupts on}
   END;

PROCEDURE Prog8bit(K: BYTE);
{
   This procedure programs the data passed to it in the array Dbyte 8 bits
   at a time starting at the address location passed to it in TA1.
}

   VAR
      M,N           : WORD;
      I,J,X         : BYTE;
      ProgSuccess   : BOOLEAN;
      ProgError     : BOOLEAN;

   BEGIN
      Numtrys := 1;
      E1 := 0;
      E2 := 0;
      ProgSuccess := NumByt = 0;
      ProgError   := FALSE;
      IF B <> 100 THEN
          if not ProgSuccess then Writeln('Touch device to program 8 bits at a time.')
                             else writeln('Programming of 0 bytes complete.');
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      REPEAT
         IF IsEPROM(K) THEN BEGIN
            Wbyte(CmdTyp);
            WByte(TA1);
            WByte(0);
            WByte(Dbyte[TA1]);
            X := TouchByte($FF);
            CRC8:= 0;
            DOWCRC(CmdTyp);
            DOWCRC(TA1);
            DOWCRC(0);
            DOWCRC(Dbyte[TA1]);
            IF CRC8 = X THEN BEGIN
               PulWidth(Propw);       {Apply program pulse on RTS}
               BYT[TA1] := TouchByte($FF);       {Read EPROM Data}
               J := 1;
               IF BYT[TA1] <> Dbyte[TA1] THEN BEGIN
                  ProgError := TRUE;
                  Numtrys := Numtrys +1;
                  E2 := E2 + 1;
               END ELSE BEGIN
                  IF NumByt = 1 THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                     IF B <> 100 THEN BEGIN
                        Writeln('Programming Completed in ',Numtrys,
                             ' attempt(s).');
                        GoodBeep;
                     END;
                     IF Diag THEN BEGIN
                        Writeln(E1,
                             ' CRC Error(s) Encountered During Programming.');
                        Writeln(E2,
                         ' Readback Error(s) Encountered During Programming.');
                        CalcTime;
                     END;
                     ProgSuccess := TRUE;
                  END ELSE BEGIN
                     Dec(NumByt);
                     ProgError := FALSE;
                     WHILE (NumByt >0) AND NOT ProgError DO BEGIN
                        Inc(TA1);
                        CRC8 := TA1;
                        Wbyte(Dbyte[TA1]);
                        DOWCRC(Dbyte[TA1]);
                        X := TouchByte($FF);
                        IF CRC8 = X THEN BEGIN
                           PulWidth(Propw);         {Apply Program Pulse}
                           BYT[TA1] := TouchByte($FF);  {Read EPROM Data}
                           IF BYT[TA1] <> Dbyte[TA1] THEN BEGIN
                              ProgError := TRUE;
                              Numtrys := Numtrys +1;
                              E2 := E2 +1;
                           END ELSE BEGIN
                              Dec(NumByt);
                              IF NumByt=0 THEN BEGIN
                                 IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                                 IF B <> 100 THEN BEGIN
                                    Writeln( 'Programming Completed in ',
                                         Numtrys,' attempt(s).');
                                    GoodBeep;
                                 END;
                                 IF Diag THEN BEGIN
                                    Writeln(E1,
                              ' CRC Error(s) Encountered During Programming.'
                                         );
                                    Writeln(E2,
                         ' Readback Error(s) Encountered During Programming.'
                                         );
                                    CalcTime;
                                 END;
                                 ProgSuccess := TRUE;
                              END;
                           END;

                        END ELSE BEGIN
                           E1 := E1 + 1;
                           Numtrys := Numtrys + 1;
                           ProgError := TRUE;
                        END;
                     END;
                  END;
               END;

            END ELSE BEGIN
               E1 := E1 +1;
               Numtrys := Numtrys + 1;
               ProgError := TRUE;
            END;
         END;
      UNTIL ProgSuccess OR (Numtrys = 20) or KeyPressed;
      IF NOT ProgSuccess THEN BEGIN
         IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
         IF B <> 100 THEN BEGIN
            Writeln('Unable to Program after ',Numtrys-1,' attempts.');
            Badbeep;
         END;
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Error(s) Encountered During Programming.') ;
            Writeln(E2, ' Readback Error(s) Encountered During Programming.');
            CalcTime;
         END;
      END;
   END;                         {End of Procedure Prog8Bit}


PROCEDURE Prog4bit(K: BYTE);
{
   This procedure programs the data passed to it in the array Dbyte 4 bits
   at a time starting at the address location passed to it in TA1.
}

   VAR
      M,N           : WORD;
      I,J,X         : BYTE;
      ProgSuccess   : BOOLEAN;
      ProgError     : BOOLEAN;
      NumBytSav     : WORD;
      Data          : BYTE;

   BEGIN
      Numtrys := 1;
      E1 := 0;
      E2 := 0;
      ProgSuccess := NumByt = 0;
      ProgError   := FALSE;
      M           := 0;
      TA1Sav      := TA1;
      NumBytSav   := NumByt;
      IF B <> 100 THEN Writeln('Touch device to program 4 bits at a time.');
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      WHILE (M<2) AND (Numtrys<21) and (not keypressed) DO BEGIN
         TA1 := TA1Sav;
         NumByt := NumBytSav;
         WHILE (NumByt>0) AND (Numtrys<21) and (not keypressed) DO BEGIN
            IF IsEPROM(K) THEN BEGIN
               Data := Dbyte[TA1] OR (NOT($F0 ShR (M*4)));
               Wbyte(CmdTyp);
               WByte(TA1);
               WByte(0);
               WByte(Data);
               X := TouchByte($FF);
               CRC8:= 0;
               DOWCRC(CmdTyp);
               DOWCRC(TA1);
               DOWCRC(0);
               DOWCRC(Data);
               IF CRC8 = X THEN BEGIN
                  PulWidth(Propw);          {Apply program pulse on RTS}
                  BYT[TA1] := TouchByte($FF);          {Read EPROM Data}
                  IF (BYT[TA1] ShR ((1-M)*4)) <> (Dbyte[TA1] ShR ((1-M)*4))
                       THEN BEGIN
                     ProgError := TRUE;
                     Numtrys := Numtrys +1;
                     E2 := E2+1;
                  END ELSE BEGIN
                     IF (NumByt = 1) AND (M=1) THEN BEGIN
                        IF Diag THEN GetTime(Hour,Min2,Sec2, Hun2);
                        IF B <> 100 THEN BEGIN
                           Writeln('Programming Completed in ',Numtrys,
                                ' attempt(s).');
                           GoodBeep;
                        END;
                        IF Diag THEN BEGIN
                           Writeln(E1,
                            ' CRC Error(s) Encountered During Programming.');
                           Writeln(E2,
                       ' Readback Error(s) Encountered During Programming.');
                           CalcTime;
                        END;
                        ProgSuccess := TRUE;
                        Dec(NumByt);
                     END ELSE BEGIN
                        Dec(NumByt);
                        ProgError := FALSE;
                        WHILE (NumByt >0) AND NOT ProgError DO BEGIN
                           Inc(TA1);
                           CRC8 := TA1;
                           Data := Dbyte[TA1] OR (NOT($F0 ShR (M*4)));
                           Wbyte(Data);
                           DOWCRC(Data);
                           X := TouchByte($FF);
                           IF CRC8 = X THEN BEGIN
                              PulWidth(Propw);       {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF);  {Read EPROM Data}
                              IF (BYT[TA1] ShR ((1-M)*4)) <> (Dbyte[ TA1] ShR
                                   ((1-M)*4)) THEN BEGIN
                                 ProgError := TRUE;
                                 Numtrys := Numtrys +1;
                                 E2 := E2 +1;
                              END ELSE BEGIN
                                 Dec(NumByt);
                                 IF (NumByt=0) AND (M=1) THEN BEGIN
                                    IF Diag THEN GetTime(Hour, Min2,
                                         Sec2,Hun2);
                                    IF B <> 100 THEN BEGIN
                                       Writeln( 'Programming Completed in ' ,
                                            Numtrys,' attempt(s).');
                                       GoodBeep;
                                    END;
                                    IF Diag THEN BEGIN
                                       Writeln(E1,
                            ' CRC Error(s) Encountered During Programming.');
                                       Writeln(E2,
                       ' Readback Error(s) Encountered During Programming.');
                                       CalcTime;
                                    END;
                                    ProgSuccess := TRUE;
                                 END;
                              END;

                           END ELSE BEGIN
                              E1 := E1 + 1;
                              Numtrys := Numtrys + 1;
                              ProgError := TRUE;
                           END;
                        END;
                     END;
                  END;

               END ELSE BEGIN
                  E1 := E1 + 1;
                  Numtrys := Numtrys + 1;
                  ProgError := TRUE;
               END;
            END;
         END;             {Number of partial bytes programmed on each loop}
         M := M +1;
      END;                      {Number of loops thru all bytes}
      IF NOT ProgSuccess THEN BEGIN
         IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
         IF B <> 100 THEN BEGIN
            Writeln('Unable to Program after ',Numtrys-1,' attempts.');
            BadBeep;
         END;
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Error(s) Encountered During Programming.');
            Writeln(E2, ' Readback Error(s) Encountered During Programming.');
            CalcTime;
         END;
      END;
   END;                         {End of Procedure Prog4Bit}


PROCEDURE Prog2bit(K: BYTE);
{
   This procedure programs the data passed to it in the array Dbyte 2 bits
   at a time starting at the address location passed to it in TA1.
}

   VAR
      M,N           : WORD;
      I,J,X         : BYTE;
      ProgSuccess   : BOOLEAN;
      ProgError     : BOOLEAN;
      NumBytSav     : WORD;
      Data          : BYTE;

   BEGIN
      Numtrys := 1;
      E1 := 0;
      E2 := 0;
      ProgSuccess := NumByt = 0;
      ProgError   := FALSE;
      M           := 0;
      TA1Sav      := TA1;
      NumBytSav   := NumByt;
      IF B <> 100 THEN Writeln('Touch device to program 2 bits at a time.');
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      WHILE (M<4) AND (Numtrys<21) and (not keypressed) DO BEGIN
         TA1 := TA1Sav;
         NumByt := NumBytSav;
         WHILE (NumByt>0) AND (Numtrys<21) and (not keypressed) DO BEGIN
            IF IsEPROM(K) THEN BEGIN
               Data := Dbyte[TA1] OR (NOT($C0 ShR (M*2)));
               Wbyte(CmdTyp);
               WByte(TA1);
               WByte(0);
               WByte(Data);
               X := TouchByte($FF);
               CRC8:= 0;
               DOWCRC(CmdTyp);
               DOWCRC(TA1);
               DOWCRC(0);
               DOWCRC(Data);
               IF CRC8 = X THEN BEGIN
                  PulWidth(Propw);          {Apply program pulse on RTS}
                  BYT[TA1] := TouchByte($FF);          {Read EPROM Data}
                  IF (BYT[TA1] ShR ((3-M)*2)) <> (Dbyte[TA1] ShR ((3-M)*2))
                       THEN BEGIN
                     ProgError := TRUE;
                     Numtrys := Numtrys + 1;
                     E2 := E2+1;
                  END ELSE BEGIN
                     IF (NumByt = 1) AND (M=3) THEN BEGIN
                        IF Diag THEN GetTime(Hour,Min2,Sec2, Hun2);
                        IF B <> 100 THEN BEGIN
                           Writeln('Programming Completed in ',Numtrys,
                                ' attempt(s).');
                           GoodBeep;
                        END;
                        IF Diag THEN BEGIN
                           Writeln(E1,
                              ' CRC Error(s) Encountered During Programming.');
                           Writeln(E2,
                         ' Readback Error(s) Encountered During Programming.');
                           CalcTime;
                        END;
                        ProgSuccess := TRUE;
                        Dec(NumByt);
                     END ELSE BEGIN
                        Dec(NumByt);
                        ProgError := FALSE;
                        WHILE (NumByt >0) AND NOT ProgError DO BEGIN
                           Inc(TA1);
                           CRC8 := TA1;
                           Data := Dbyte[TA1] OR (NOT($C0 ShR (M*2)));
                           Wbyte(Data);
                           DOWCRC(Data);
                           X := TouchByte($FF);
                           IF CRC8 = X THEN BEGIN
                              PulWidth(Propw);        {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF); {Read EPROM Data}
                              IF (BYT[TA1] ShR ((3-M)*2)) <> (Dbyte[ TA1] ShR
                                   ((3-M)*2)) THEN BEGIN
                                 ProgError := TRUE;
                                 Numtrys := Numtrys +1;
                                 E2 := E2 +1;
                              END ELSE BEGIN
                                 Dec(NumByt);
                                 IF (NumByt=0) AND (M=3) THEN BEGIN
                                    IF Diag THEN GetTime(Hour, Min2,
                                         Sec2,Hun2);
                                    IF B <> 100 THEN BEGIN
                                       Writeln( 'Programming Completed in ' ,
                                            Numtrys,' attempt(s).');
                                       GoodBeep;
                                    END;
                                    IF Diag THEN BEGIN
                                       Writeln(E1,
                            ' CRC Error(s) Encountered During Programming.');
                                       Writeln(E2,
                       ' Readback Error(s) Encountered During Programming.');
                                       CalcTime;
                                    END;
                                    ProgSuccess := TRUE;
                                 END;
                              END;

                           END ELSE BEGIN
                              E1 := E1 + 1;
                              Numtrys := Numtrys + 1;
                              ProgError := TRUE;
                           END;
                        END;
                     END;
                  END;

               END ELSE BEGIN
                  E1 := E1 +1;
                  Numtrys := Numtrys + 1;
                  ProgError := TRUE;
               END;
            END;
         END;             {Number of partial bytes programmed on each loop}
         M := M +1;
      END;                      {Number of loops thru all bytes}
      IF NOT ProgSuccess THEN BEGIN
         IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
         IF B <> 100 THEN BEGIN
            Writeln('Unable to Program after ',Numtrys-1,' attempts.');
            BadBeep;
         END;
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Error(s) Encountered During Programming.') ;
            Writeln(E2, ' Readback Error(s) Encountered During Programming.');
            CalcTime;
         END;
      END;

   END;                         {End of Procedure Prog2Bit}


PROCEDURE Prog1bit(K: BYTE);
{
   This procedure programs the data passed to it in the array Dbyte 1 bit
   at a time starting at the address location passed to it in TA1.
}

   VAR
      M,N           : WORD;
      I,J,X         : BYTE;
      ProgSuccess   : BOOLEAN;
      ProgError     : BOOLEAN;
      NumBytSav     : WORD;
      Data          : BYTE;

   BEGIN
      Numtrys := 1;
      E1 := 0;
      E2 := 0;
      ProgSuccess := FALSE;
      ProgSuccess := NumByt = 0;
      M           := 0;
      TA1Sav      := TA1;
      NumBytSav   := NumByt;
      IF B <> 100 THEN Writeln('Touch device to program 1 bit at a time.');
      IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
      WHILE (M<8) AND (Numtrys<21) and (not keypressed) DO BEGIN
         TA1 := TA1Sav;
         NumByt := NumBytSav;
         WHILE (NumByt>0) AND (Numtrys<21) and (not keypressed) DO BEGIN
            IF IsEPROM(K) THEN BEGIN
               Data := Dbyte[TA1] OR (NOT($80 ShR M));
               Wbyte(CmdTyp);
               WByte(TA1);
               WByte(0);
               WByte(Data);
               X := TouchByte($FF);
               CRC8:= 0;
               DOWCRC(CmdTyp);
               DOWCRC(TA1);
               DOWCRC(0);
               DOWCRC(Data);
               IF CRC8 = X THEN BEGIN
                  PulWidth(Propw);          {Apply program pulse on RTS}
                  BYT[TA1] := TouchByte($FF);          {Read EPROM Data}
                  IF (BYT[TA1] ShR (7-M)) <> (Dbyte[TA1] ShR (7-M)) THEN
                       BEGIN
                     ProgError := TRUE;
                     Numtrys := Numtrys +1;
                     E2 := E2+1;
                  END ELSE BEGIN
                     IF (NumByt = 1) AND (M=7) THEN BEGIN
                        IF Diag THEN GetTime(Hour,Min2,Sec2, Hun2);
                        IF B <> 100 THEN BEGIN
                           Writeln('Programming Completed in ',Numtrys,
                                ' attempt(s).');
                           GoodBeep;
                        END;
                        IF Diag THEN BEGIN
                           Writeln(E1,
                            ' CRC Error(s) Encountered During Programming.');
                           Writeln(E2,
                       ' Readback Error(s) Encountered During Programming.');
                           CalcTime;
                        END;
                        ProgSuccess := TRUE;
                        Dec(NumByt);
                     END ELSE BEGIN
                        Dec(NumByt);
                        ProgError := FALSE;
                        WHILE (NumByt >0) AND NOT ProgError DO BEGIN
                           Inc(TA1);
                           CRC8 := TA1;
                           Data := Dbyte[TA1] OR (NOT($80 ShR M));
                           Wbyte(Data);
                           DOWCRC(Data);
                           X := TouchByte($FF);
                           IF CRC8 = X THEN BEGIN
                              PulWidth(Propw);        {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF); {Read EPROM Data}
                              IF (BYT[TA1] ShR (7-M)) <> (Dbyte[TA1] ShR
                                   (7 - M)) THEN BEGIN
                                 ProgError := TRUE;
                                 Numtrys := Numtrys +1;
                                 E2 := E2 +1;
                              END ELSE BEGIN
                                 Dec(NumByt);
                                 IF (NumByt=0) AND (M=7) THEN BEGIN
                                    IF Diag THEN GetTime(Hour, Min2,
                                         Sec2,Hun2);
                                    IF B <> 100 THEN BEGIN
                                       Writeln( 'Programming Completed in ' ,
                                            Numtrys,' attempt(s).');
                                       GoodBeep;
                                    END;
                                    IF Diag THEN BEGIN
                                       Writeln(E1,
                            ' CRC Error(s) Encountered During Programming.');
                                       Writeln(E2,
                       ' Readback Error(s) Encountered During Programming.');
                                       CalcTime;
                                    END;
                                    ProgSuccess := TRUE;
                                 END;
                              END;

                           END ELSE BEGIN
                              E1 := E1 + 1;
                              Numtrys := Numtrys + 1;
                              ProgError := TRUE;
                           END;
                        END;
                     END;
                  END;

               END ELSE BEGIN
                  E1 := E1 +1;
                  Numtrys := Numtrys + 1;
                  ProgError := TRUE;
               END;
            END;
         END;         {Number of partial bytes programmed on each loop}
         M := M +1;
      END;                      {Number of loops thru all bytes}
      IF NOT ProgSuccess THEN BEGIN
         IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
         IF B <> 100 THEN BEGIN
            Writeln('Unable to Program after ',Numtrys-1,' attempts.');
            BadBeep;
         END;
         IF Diag THEN BEGIN
            Writeln(E1,' CRC Error(s) Encountered During Programming.') ;
            Writeln(E2, ' Readback Error(s) Encountered During Programming.');
            CalcTime;
         END;
      END;
   END;                         {End of Procedure Prog1Bit}

PROCEDURE FindBlank(K: BYTE; VAR NotBlank: BOOLEAN; VAR UnPro, UnProTA1: WORD);
{
   This procedure checks to see if a sufficient number of unprogrammed cells
   exist starting at the selected TA1 address. A worst case maximum of 8 are
   required for the serial port calibration operation. If a sufficient amount
   are found, the variable NotBlank is returned as False. If NotBlank is
   True, an attempt is made to indicate the first available address where
   8 consecutive unprogrammed cells might be found.
}

   VAR
      ReadSuccess : BOOLEAN;
      Endloop     : BOOLEAN;
      CRCSav      : ARRAY[0..4] OF BYTE;
      I,J         : BYTE;

   BEGIN
      NotBlank:=FALSE;
      UnPRo:=100;
      UnProTa1:=2041;
      PGNM:=0;
      BYCNT:=0;
      Numtrys:=1;
      E1:=0;
      ReadSuccess:=FALSE;
      Endloop:= FALSE;
      IF TouchReset(K) THEN BEGIN
         IF Diag THEN GetTime(Hour,Min1,Sec1,Hun1);
         WHILE (NumTrys<21) AND NOT ReadSuccess DO BEGIN
            IF TouchReset(K) THEN BEGIN
               WByte($CC);
               Wbyte($F0);
               WByte(0);
               WByte(0);
               CRCSav[4] := TouchByte($FF);
               CRC8 := 0;
               DOWCRC($F0);
               DOWCRC(0);
               DOWCRC(0);
               IF CRC8 = CRCSav[4] THEN BEGIN
                  CRC8 := 0;
                  TA1 := 0;
                  FOR BYCNT := 0 TO 127 DO BEGIN
                     BYT[TA1] := RByte;
                     Inc(TA1);
                  END;
                  CRCSav[0] := TouchByte($FF);
                  IF CRCSav[0] = CRC8 THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                     ReadSuccess := TRUE;
                     GoodBeep;
                     IF Diag THEN BEGIN
                        Writeln('Reading completed in ',Numtrys,
                             ' attempt(s).');
                        CalcTime;
                     END;
                     UnProTA1:=0;
                     UnPro:=0;
                     NotBlank:=TRUE;
                     FOR J:=0 TO 7 DO UnProTA1 := UnProTA1 + BYT[ TA1Sav+ J];
                     IF UnProTA1 = 2040 THEN NotBlank:=FALSE ELSE BEGIN
                        NotBlank:=TRUE;
                        Writeln(
                             'Unable to Calibrate Starting at Address TA1=' ,
                             TA1Sav,'.');
                        Writeln(
      'Insufficient Number of Consecutive Unprogrammed Bytes (8 required).');
                        J:=0;
                        WHILE (J<120) AND (UnPro<2040) DO BEGIN
                           UnPro:=0;
                           FOR I:=0 TO 7 DO UnPro:=UnPro + BYT[J+I];
                           Inc(J);
                        END;
                        IF UnPro<2040 THEN BEGIN
                           Write(
                           'Unable to Find Eight Consecutive Unprogrammed ');
                           Writeln('Bytes Within Device.');
                        END ELSE BEGIN
                           Writeln(
        'Note: At Least One Block of Eight Consecutive Unprogrammed Bytes ');
                           Writeln('Can Be Found at TA1=',J-1,'.');
                        END;
                     END;
                  END ELSE BEGIN
                     E1:=E1+1;
                     Numtrys:=Numtrys+1;
                  END;
               END ELSE BEGIN
                  E1:=E1+1;
                  Numtrys:=Numtrys+1;
               END;
            END;
         END;
         IF Numtrys>20 THEN BEGIN
            GetTime(Hour,Min2,Sec2,Hun2);
            BadBeep;
            Writeln('Unable to read after ',Numtrys-1,' attempt(s).');
            IF Diag THEN BEGIN
               Writeln(E1,' CRC Errors Encountered during Reading.');
               CalcTime;
            END;
            Writeln;
         END;
      END;
   END;                         {End of Procedure FindBlank}

VAR
   FC, I,J,K,X : BYTE;
   M, N        : WORD;
   C, NewCh    : CHAR;
   ISt, Str    : STRING;
   L           : LongInt;
   CRCSav      : ARRAY[0..4] OF BYTE;
   SP          : ARRAY[1..4] OF WORD Absolute $40:0;
   TI          : BYTE;
   LL          : LongInt;
   YR, MO, DA  : LongInt;
   HR, MI, SE  : BYTE;
   Data        : BYTE;
   NewSt       : STRING;
   Test,Endloop: BOOLEAN;
   ReadSuccess : BOOLEAN;
   ReadError   : BOOLEAN;
   AB          : BOOLEAN;
   BC          : byte;
   RedPGNM     : BYTE;
   NumBytSav   : WORD;
   UnPro       : WORD;
   UnProTA1    : WORD;
   NotBlank    : BOOLEAN;
   Output      : BOOLEAN;

CONST
   NumPass = [1,2,4,8];
   MaxReTry = 20;

var StByte: array [0..7] of byte;

function RdStatus: boolean;
var BC, CRCSav: byte;
begin
    RdStatus := false;
    if TouchReset(K) then begin
        WByte($CC);    WByte($AA);    WByte(0);    WByte(0);
        CRCSav := TouchByte($FF);    CRC8 := 0;
        DOWCRC($AA);    DOWCRC(0);    DOWCRC(0);
        if CRC8 = CRCSav then begin
            CRC8 := 0;    BC := 0;
            for BC := 0 to 7 do begin
                StByte[BC] := RByte;
            end;
            CRCSav := TouchByte($FF);
            if CRCSav = CRC8 then RdStatus := true;
        end;
    end;
end;

function RdStatusRT: boolean; { try to read DS1982 status, re-try if needed }
var TryNum: byte;
    Success: boolean;
begin
    Success := false;                        { no success yet }
    TryNum := 0;                             { no attempts have been made }
    repeat
        Success := RdStatus;                 { attempt to read status bytes }
        inc(TryNum);                         { one more attempt has been made }
    until Success or (TryNum > MaxRetry);
    RdStatusRT := Success;
end;

BEGIN
   ClrScr;
   Writeln;
   writeln('              DS1982/DS2502 DEMO Version 0.12');
   writeln;
   writeln('          Copyright (C) 1994 Dallas Semiconductor.');
   writeln;
   K := 1;                      { Default to COM1 }
   Pass := 8;                   { Default to byte programming}
   Propw := 1000;               { Default to 1msec program pulse}
   Diag := FALSE;               { Default turns diagnostics off}
   IF ParamCount > 0 THEN BEGIN {$R+}
      Val(ParamStr(1), L, N);   { Get COM port number from cmd line }
                                {$R-}
      IF (N > 0) OR (L < 1) OR (L > 4) THEN BEGIN
         BadBeep;
         Writeln('INCORRECT COMMAND LINE PARAMETER 1');
         Halt;
      END ELSE K := L;
      IF ParamCount > 1 THEN BEGIN
                                {$R+}
         Val(ParamStr(2), L, N);{ Get number of passes from cmd line }
                                {$R-}
         IF (N = 0) AND (L IN (NumPass)) THEN BEGIN
            Pass := L;
         END ELSE BEGIN
            Writeln('INCORRECT COMMAND LINE PARAMETER 2');
            BadBeep;
            Halt;
         END;
         IF ParamCount > 2 THEN BEGIN
                                {$R+}
            Val(ParamStr(3), L, N); { Get prog pulse width from cmd line }
                                {$R-}
            IF (N > 0) OR (L < 500) OR (L > 100000) THEN BEGIN
               BadBeep;
               Writeln('INCORRECT COMMAND LINE PARAMETER 3');
               Writeln(
                    'PROGRAM PULSE WIDTH MUST BE IN THE RANGE 500-100000.' );
               Halt;
            END ELSE PROPW := L;
            IF ParamCount >3 THEN BEGIN
               Diag := (ParamStr(4)='Y') OR (ParamStr(4)='y');
               IF NOT Diag THEN BEGIN
                  BadBeep;
                  Writeln('INCORRECT COMMAND LINE PARAMETER 4');
                  Writeln('TYPE ''Y'' TO ENABLE DIAGNOSTIC MODE.');
                  Halt;
               END;
            END;
         END;
      END;
   END;
   IF SP[K] > 0 THEN BEGIN
      REPEAT
         Writeln;
         Writeln(
'(0) Exit                  (1) Read ROM Loop        (2) Read EPROM w/ Page CRC'
              );
         Writeln(
    '(3) Read EPROM Memory     (4) Read EPROM Status    (5) Write EPROM Memory');
         Writeln(
  '(6) Write EPROM Status    (7) Program EPROM w/Text (8) Read EPROM Text');
         Writeln(
'(9) Copy RAM to EPROM     (10) Copy EPROM to EPROM (11) Write Protect Pages'
              );
         IF Diag THEN Writeln(
'(12) Set Page Redirection (13) Calibrate COM Port  (100) Program EPROM Loop'
              )
         ELSE Writeln( '(12) Set Page Redirection (13) Calibrate COM Port');
         Test := FALSE;
         WHILE NOT Test DO BEGIN
            Write('ENTER OPTION (0 to Exit): ');
            Readln(Str);
            IF Length(Str)=0 THEN BEGIN
               BadBeep;
               Writeln( 'Please select a value from the menu choices.');
            END ELSE BEGIN       {$R+}
               Val(Str, L, N);
                             {$R-}
               IF (N > 0) OR (L < 0) OR (L > 100) THEN BEGIN
                  BadBeep;
                  Writeln( 'Please select a value from the menu choices.');
               END ELSE BEGIN
                  B:= L;
                  Test := TRUE;
               END;
            END;
         END;
         CASE B OF
            0: BEGIN
               Writeln;
            END;

            1: BEGIN
               Writeln( 'Touch device to Read ROM - Type <ESC> to exit' )
                    ;
               Writeln;
               Writeln( ' CRC |      Serial Number   | FC  Device Type' );
               Endloop := FALSE;
               REPEAT
                  IF TouchReset(K) THEN BEGIN
                     WByte($33);
                     X := TouchByte($FF);
                     IF (X > 0) AND (X < $FF) THEN BEGIN
                        FC := X;
                        CRC8 := 0;
                        DOWCRC(X);
                        ROM[0] := X;
                        FOR I := 1 TO 7 DO BEGIN
                           X := TouchByte($FF);
                           DOWCRC(X);
                           ROM[I] := X;
                        END;
                        IF CRC8 = 0 THEN BEGIN
                           GoodBeep;
                           FOR I := 7 DOWNTO 0 DO Write(Hex(ROM[ I]) : 4);
                           CASE ROM[0] OF
                              $01: Writeln('    (DS1990A)');
                              $02: Writeln('    (DS1991)');
                              $04: Writeln('    (DS1994)');
                              $05: Writeln('    (DS2405)');
                              $06: Writeln('    (DS1993)');
                              $08: Writeln('    (DS1992)');
                              $09: Writeln('    (DS1982)');
                           ELSE
                              Writeln;
                           END;
                           I := 0;
                           REPEAT
                              IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
                              IF KeyPressed AND (Readkey=#27) THEN
                                   Endloop := TRUE;
                             {terminates loop if part is in a socket and
                                cannot be disconnected}
                           UNTIL (I = 100) OR Endloop;
                        END;
                     END;
                  END;
               UNTIL KeyPressed AND (Readkey=#27) OR Endloop;
            END;

            2: BEGIN
               Writeln(
                'Touch device to Read EPROM w/ Page CRC - Type <ESC> to exit ');
               REPEAT
                  PGNM:=0;
                  BYCNT:=0;
                  Numtrys:=1;
                  E1:=0;
                  TA1:=0;
                  ReadSuccess:=FALSE;
                  Endloop:=FALSE;
                  Output:=TRUE;
                  IF TouchReset(K) THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                     ReadEPROMcrc(K,Output,ReadSuccess);
                     IF Numtrys>20 THEN BEGIN
                        GetTime(Hour,Min2,Sec2,Hun2);
                        BadBeep;
                        Writeln;
                        Writeln('Unable to read after ',Numtrys- 1,
                             ' attempt(s).');
                        IF Diag THEN BEGIN
                           Writeln(E1,
                                ' CRC Errors Encountered during Reading.');
                           CalcTime;
                        END;
                        Writeln;
                     END;
                     I := 0;
                     REPEAT
                        IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
                        IF KeyPressed AND (Readkey=#27) THEN Endloop :=
                             TRUE;
                             {terminates loop if part is in a socket and
                                cannot be disconnected}
                     UNTIL (I = 100) OR Endloop;
                  END;
               UNTIL KeyPressed AND (Readkey=#27) OR Endloop;
            END;

            3: BEGIN
               Writeln(
                  'Touch device to Read EPROM Memory - Type <ESC> to exit ');
               REPEAT
                  PGNM:=0;
                  BYCNT:=0;
                  Numtrys:=1;
                  E1:=0;
                  TA1:=0;
                  ReadSuccess:=FALSE;
                  Endloop:= FALSE;
                  IF TouchReset(K) THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                     WHILE (NumTrys<21) AND NOT ReadSuccess DO BEGIN
                        IF TouchReset(K) THEN BEGIN
                           WByte($CC);
                           Wbyte($F0);
                           WByte(0);
                           WByte(0);
                           CRCSav[4] := TouchByte($FF);
                           CRC8 := 0;
                           DOWCRC($F0);
                           DOWCRC(0);
                           DOWCRC(0);
                           IF CRC8 = CRCSav[4] THEN BEGIN
                              CRC8 := 0;
                              TA1:= 0;
                              FOR BYCNT := 0 TO 127 DO BEGIN
                                 BYT[TA1] := RByte;
                                 Inc(TA1);
                              END;
                              CRCSav[0] := TouchByte($FF);
                              IF CRCSav[0] = CRC8 THEN BEGIN
                                 IF Diag THEN GetTime(Hour,Min2,Sec2,Hun2);
                                 ReadSuccess := TRUE;
                                 GoodBeep;
                                 Writeln;
                                 Writeln('Initial CRC = ',Hex(CRCSav[ 4]));
                                 writeln('     0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F');
                                 FOR PGNM:=0 TO 3 DO BEGIN
                                    TA1:=PGNM*32;
                                    J:=0;
                                    FOR J:=0 TO 1 DO BEGIN
                                       write(Hex(TA1), ' ');
                                       FOR BYCNT := 0 TO 15 DO BEGIN
                                          Write(Hex(BYT[TA1]):3);
                                          Inc(TA1);
                                       END;
                                       Writeln;
                                    END;
                                 END;
                                 Writeln('CRC for 1K Memory: ', Hex(
                                      CRCSav[0]));
                                 IF Diag THEN BEGIN
                                    Writeln( 'Reading completed in ',
                                         Numtrys,' attempt(s).');
                                    CalcTime;
                                 END;
                              END ELSE BEGIN
                                 E1:=E1+1;
                                 Numtrys:=Numtrys+1;
                              END;

                           END ELSE BEGIN
                              E1:=E1+1;
                              Numtrys:=Numtrys+1;
                           END;
                        END;
                     END;
                     IF Numtrys>20 THEN BEGIN
                        GetTime(Hour,Min2,Sec2,Hun2);
                        BadBeep;
                        Writeln('Unable to read after ',Numtrys- 1,
                             ' attempt(s).');
                        IF Diag THEN BEGIN
                           Writeln(E1,
                                ' CRC Errors Encountered during Reading.');
                           CalcTime;
                        END;
                        Writeln;
                     END;
                     I := 0;
                     REPEAT
                        IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
                        IF KeyPressed AND (Readkey=#27) THEN Endloop :=
                             TRUE;
                             {terminates loop if part is in a socket and
                                cannot be disconnected}
                     UNTIL (I = 100) OR Endloop;
                  END;
               UNTIL KeyPressed AND (Readkey=#27) OR Endloop;
            END;

            4: BEGIN
               Writeln('Touch device to Read EPROM Status - Type <ESC> to exit');
               REPEAT
                  PGNM:=0;
                  BYCNT:=0;
                  Numtrys:=1;
                  E1:=0;
                  TA1:=0;
                  ReadSuccess:=FALSE;
                  Endloop:=FALSE;
                  IF TouchReset(K) THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                     WHILE (NumTrys<21) AND NOT ReadSuccess DO BEGIN
                        IF TouchReset(K) THEN BEGIN
                           WByte($CC);
                           Wbyte($AA);
                           WByte(TA1);
                           WByte(0);
                           CRCSav[4] := TouchByte($FF);
                           CRC8 := 0;
                           DOWCRC($AA);
                           DOWCRC(TA1);
                           DOWCRC(0);
                           IF CRC8 = CRCSav[4] THEN BEGIN
                              CRC8 := 0;
                              TA1:= 0;
                              FOR BYCNT := 0 TO 7 DO BEGIN
                                 BYT[TA1] := RByte;
                                 Inc(TA1);
                              END;
                              CRCSav[0] := TouchByte($FF);
                              IF CRCSav[0] = CRC8 THEN BEGIN
                                 IF Diag THEN GetTime( Hour, Min2,
                                      Sec2,Hun2);
                                 ReadSuccess := TRUE;
                                 GoodBeep;
                                 Writeln;
                                 Writeln('Initial CRC = ',Hex( CRCSav[ 4])
                                      );
                                 writeln('  0  1  2  3  4  5  6  7');
                                 TA1:=0;
                                 FOR BYCNT := 0 TO 7 DO BEGIN
                                    Write(Hex(BYT[TA1]):3);
                                    Inc(TA1);
                                 END;
                                 Writeln;
                                 Writeln( 'CRC for Status Memory: ', Hex(
                                      CRCSav[0]));
                                 IF Diag THEN BEGIN
                                    Writeln( 'Reading completed in ',
                                         Numtrys,' attempt(s).');
                                    CalcTime;
                                 END;
                                 FOR PGNM:= 0 TO 3 DO BEGIN
                                    IF ((BYT[0] ShR PGNM) AND $01) = $01
                                         THEN Write( 'Page ', PGNM,
                                         ': NOT Write-Protected,' )
                                    ELSE Write('Page ',PGNM,
                                         ': Write-Protected,');
                                    IF BYT[PGNM+1] = $FF THEN Writeln(
                                         'NOT Redirected')
                                    ELSE Writeln( 'Redirected to Page ',
                                         Hex( NOT BYT[PGNM+1]));
                                 END;
                              END ELSE BEGIN
                                 E1:=E1+1;
                                 Numtrys:=Numtrys+1;
                              END;

                           END ELSE BEGIN
                              E1:=E1+1;
                              Numtrys:=Numtrys+1;
                           END;
                        END;
                     END;
                     IF Numtrys>20 THEN BEGIN
                        GetTime(Hour,Min2,Sec2,Hun2);
                        BadBeep;
                        Writeln('Unable to read after ',Numtrys- 1,
                             ' attempt(s).');
                        IF Diag THEN BEGIN
                           Writeln(E1,
                                ' CRC Errors Encountered during Reading.');
                           CalcTime;
                        END;
                        Writeln;
                     END;
                     I := 0;
                     REPEAT
                        IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
                        IF KeyPressed AND (Readkey=#27) THEN Endloop :=
                             TRUE;
                             {terminates loop if part is in a socket and
                                cannot be disconnected}
                     UNTIL (I = 100) OR Endloop;
                  END;
               UNTIL KeyPressed AND (Readkey=#27) OR Endloop;
            END;

            5: BEGIN
               CmdTyp:=$0F;
               Writeln('Executing Write EPROM Memory');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write('Enter value for TA1(0): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     TA1 :=0;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L > 127) THEN BEGIN
                        BadBeep;
                        Writeln('Value for TA1 must be in the range 0-127' );
                     END ELSE BEGIN
                        TA1 := L;
                        Test := TRUE;
                     END;
                  END;
               END;
               Writeln('Note: TA2 = 00h ');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write('Enter number of bytes to program(1): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     NumByt :=1;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) THEN BEGIN
                        BadBeep;
                        Writeln(
                             'Number of bytes must be in the range 0-128.');
                     END ELSE BEGIN
                        IF (L > (128 - TA1)) THEN BEGIN
                           BadBeep;
                           Writeln(
                  'Insufficent number of bytes for selected value of TA1('
                                ,TA1,').');
                           Writeln(
                                'Maximum number of bytes remaining is ' ,(
                                128-TA1),'.');
                        END ELSE BEGIN
                           NumByt := L;
                           Test := TRUE;
                        END;
                     END;
                  END;
               END;
               Test := FALSE;
               WHILE (NOT Test) and (NumByt > 0) DO BEGIN
                  Write( 'Enter value for data bytes in hex (FF): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     Data:=$FF;
                     Test := TRUE;
                  END ELSE BEGIN
                     Str:= '$' + Str;
                             {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L>255) THEN BEGIN
                        BadBeep;
                        Writeln(
                        'Value for data bytes must be in the range 00-FF.');
                     END ELSE BEGIN
                        Data:=L;
                        Test := TRUE;
                     END;
                  END;
               END;
               IF NumByt > 0 then begin
                   FOR I := TA1 TO (TA1+Numbyt-1) DO BEGIN
                      Dbyte[I] := Data;
                   END;
                   CASE Pass OF
                      8: BEGIN
                         Prog8Bit(K);
                      END;
                      4: BEGIN
                         Prog4Bit(K);
                      END;
                      2: BEGIN
                         Prog2Bit(K);
                      END;
                      1: BEGIN
                         Prog1Bit(K);
                      END;
                   END;
               END;
            END;

            6: BEGIN
               CmdTyp:=$55;
               Writeln('Executing Write EPROM Status');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write('Enter value for TA1(1): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     TA1 :=1;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L > 7) THEN BEGIN
                        BadBeep;
                        Writeln( 'Value for TA1 must be in the range 0-7');
                     END ELSE BEGIN
                        TA1 := L;
                        Test := TRUE;
                     END;
                  END;
               END;
               Writeln('Note: TA2 = 00h ');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write('Enter number of bytes to program(1): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     NumByt :=1;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) THEN BEGIN
                        BadBeep;
                        Writeln(
                             'Number of bytes must be in the range 0-8.' );
                     END ELSE BEGIN
                        IF (L > (8 - TA1)) THEN BEGIN
                           BadBeep;
                           Writeln(
                  'Insufficent number of bytes for selected value of TA1('
                                ,TA1,').');
                           Writeln(
                                'Maximum number of bytes remaining is ' ,(
                                8-TA1),'.');
                        END ELSE BEGIN
                           NumByt := L;
                           Test := TRUE;
                        END;
                     END;
                  END;
               END;
               Test := FALSE;
               WHILE (NOT Test) and (NumByt > 0) DO BEGIN
                  Write( 'Enter value for data bytes in hex (FF): ' );
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     Data:=$FF;
                     Test := TRUE;
                  END ELSE BEGIN
                     Str:= '$' + Str;
                             {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L>255) THEN BEGIN
                        BadBeep;
                        Writeln(
                        'Value for data bytes must be in the range 00-FF.');
                     END ELSE BEGIN
                        Data:=L;
                        Test := TRUE;
                     END;
                  END;
               END;
               IF NumByt > 0 then begin
                   FOR I := TA1 TO (TA1+Numbyt-1) DO Dbyte[I] := Data;
                   CASE Pass OF
                      8: BEGIN
                         Prog8Bit(K);
                      END;
                      4: BEGIN
                         Prog4Bit(K);
                      END;
                      2: BEGIN
                         Prog2Bit(K);
                      END;
                      1: BEGIN
                         Prog1Bit(K);
                      END;
                   END;
               end;
            END;

            100: BEGIN
               Writeln('Executing EPROM Program Loop');
               Writeln;
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write(
                      'Enter value in hex for program type (0F) or (55): ');
                  Readln(Str);
                  BEGIN
                     Str:= '$' + Str;
                             {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR ((L <> 15) AND (L <> 85)) THEN BEGIN
                        BadBeep;
                        Writeln(
                        'Value for program type must be 0F hex or 55 hex.');
                     END ELSE BEGIN
                        CmdTyp:=L;
                        Test := TRUE;
                     END;
                  END;
               END;
               Test:= FALSE;
               WHILE NOT Test DO BEGIN
                  Write( 'Enter value for Program Pulse Width (1000): ' );
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     PROPW :=1000;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 1) OR (L > 100000) THEN BEGIN
                        BadBeep;
                        Writeln(
                           'Value for PROPW must be in the range 1-100000');
                     END ELSE BEGIN
                        PROPW := L;
                        Test := TRUE;
                     END;
                  END;
               END;
               Test := FALSE;
               IF CmdTyp = $0F THEN BEGIN
                  WHILE NOT Test DO BEGIN
                     Write('Enter value for TA1(0): ');
                     Readln(Str);
                     IF Length(Str)=0 THEN BEGIN
                        TA1Sav:=0;
                        Test := TRUE;
                     END ELSE BEGIN
                             {$R+}
                        Val(Str, L, N);
                             {$R-}
                        IF (N > 0) OR (L < 0) OR (L > 127) THEN BEGIN
                           BadBeep;
                           Writeln(
                                'Value for TA1 must be in the range 0-127');
                        END ELSE BEGIN
                           TA1Sav:= L;
                           Test := TRUE;
                        END;
                     END;
                  END;
                  Writeln('Note: TA2 = 00h ');
                  Test := FALSE;
                  WHILE NOT Test DO BEGIN
                     Write( 'Enter number of bytes to program(1): ' );
                     Readln(Str);
                     IF Length(Str)=0 THEN BEGIN
                        NumBytSav:=1;
                        Test := TRUE;
                     END ELSE BEGIN
                             {$R+}
                        Val(Str, L, N);
                             {$R-}
                        IF (N > 0) OR (L < 0) THEN BEGIN
                           BadBeep;
                           Writeln(
                             'Number of bytes must be in the range 0-128.');
                        END ELSE BEGIN
                           IF (L > (128 - TA1Sav)) THEN BEGIN
                              BadBeep;
                              Writeln(
                  'Insufficent number of bytes for selected value of TA1('
                                   ,TA1Sav,').');
                              Writeln(
                                   'Maximum number of bytes remaining is '
                                   ,(128-TA1Sav),'.');
                           END ELSE BEGIN
                              NumBytSav:= L;
                              Test := TRUE;
                           END;
                        END;
                     END;
                  END;
               END;
               IF CmdTyp = $55 THEN BEGIN
                  WHILE NOT Test DO BEGIN
                     Write('Enter value for TA1(1): ');
                     Readln(Str);
                     IF Length(Str)=0 THEN BEGIN
                        TA1Sav:=1;
                        Test := TRUE;
                     END ELSE BEGIN
                             {$R+}
                        Val(Str, L, N);
                             {$R-}
                        IF (N > 0) OR (L < 0) OR (L > 7) THEN BEGIN
                           BadBeep;
                           Writeln(
                                'Value for TA1 must be in the range 0-7' );
                        END ELSE BEGIN
                           TA1Sav:= L;
                           Test := TRUE;
                        END;
                     END;
                  END;
                  Writeln('Note: TA2 = 00h ');
                  Test := FALSE;
                  WHILE NOT Test DO BEGIN
                     Write( 'Enter number of bytes to program(1): ' );
                     Readln(Str);
                     IF Length(Str)=0 THEN BEGIN
                        NumBytSav:=1;
                        Test := TRUE;
                     END ELSE BEGIN
                             {$R+}
                        Val(Str, L, N);
                             {$R-}
                        IF (N > 0) OR (L < 0) THEN BEGIN
                           BadBeep;
                           Writeln(
                               'Number of bytes must be in the range 0-8.');
                        END ELSE BEGIN
                           IF (L > (8 - TA1Sav)) THEN BEGIN
                              BadBeep;
                              Writeln(
                  'Insufficent number of bytes for selected value of TA1('
                                   ,TA1Sav,').');
                              Writeln(
                                   'Maximum number of bytes remaining is '
                                   ,(8-TA1Sav),'.');
                           END ELSE BEGIN
                              NumBytSav:= L;
                              Test := TRUE;
                           END;
                        END;
                     END;
                  END;
               END;
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write( 'Enter value for data bytes in hex (FF): ' );
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     Data:=$FF;
                     Test := TRUE;
                  END ELSE BEGIN
                     Str:= '$' + Str;
                             {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L>255) THEN BEGIN
                        BadBeep;
                        Writeln(
                        'Value for data bytes must be in the range 00-FF.');
                     END ELSE BEGIN
                        Data:=L;
                        Test := TRUE;
                     END;
                  END;
               END;
               REPEAT
                  TA1 := TA1Sav;
                  NumByt := NumBytSav;
                  FOR I := TA1 TO (TA1+Numbyt-1) DO Dbyte[I] := Data;
                  CASE Pass OF
                     8: BEGIN
                        Prog8Bit(K);
                     END;
                     4: BEGIN
                        Prog4Bit(K);
                     END;
                     2: BEGIN
                        Prog2Bit(K);
                     END;
                     1: BEGIN
                        Prog1Bit(K);
                     END;
                  END;
               UNTIL KeyPressed AND (Readkey=#27);
            END;
            7: BEGIN
               CmdTyp:=$0F;
               Writeln('Executing Program EPROM w/ text');
               TextIn;
               NumByt:=Dbyte[0]+3;
               Writeln('Number of characters:',Dbyte[0]);
               Writeln;
               Writeln('Please Touch DS1982.');
               Writeln;
               TA1 := 0;
               TA2 := 0;
               CASE Pass OF
                  8: BEGIN
                     Prog8Bit(K);
                  END;
                  4: BEGIN
                     Prog4Bit(K);
                  END;
                  2: BEGIN
                     Prog2Bit(K);
                  END;
                  1: BEGIN
                     Prog1Bit(K);
                  END;
               END;
            END;

            8: BEGIN
               Writeln('Touch device to Read EPROM Text - Type <ESC> to exit');
               Endloop:=FALSE;
               MyMission:='DEMO'+Chr(0);
               REPEAT
                  IF TouchReset(K) THEN BEGIN
                     Writeln;
                     REadEPROM(K,READSuccess);
                     I := 0;
                     REPEAT
                        IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
                        IF (KeyPressed AND (Readkey=#27)) THEN Endloop :=
                             TRUE;
                             {terminates loop if part is in a socket and
                                cannot be disconnected}
                     UNTIL (I = 100) OR Endloop;
                  END;
               UNTIL KeyPressed AND (Readkey=#27) OR Endloop;
            END;

            9: BEGIN
               Writeln('Executing Copy RAM to EPROM - Type <ESC> to exit');
               CmdTyp:=$0F;
               Test:=TRUE;
               Writeln('Please Touch Source Device.');
               ReadRam(K,ReadSuccess);
               I := 0;
               REPEAT
                  IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop if probe is lifted from
                                Touch Memory}
               UNTIL (I = 100) or KeyPressed;
               IF ReadSuccess THEN BEGIN
                  REPEAT
                     Write('Please Touch DS1982 - ');
                     TA1 := 0;
                     TA2 := 0;
                     NumByt:=128;
                     FOR I:=0 TO 127 DO Dbyte[I]:=RAM[I];
                     CASE Pass OF
                        8: BEGIN
                           Prog8Bit(K);
                        END;
                        4: BEGIN
                           Prog4Bit(K);
                        END;
                        2: BEGIN
                           Prog2Bit(K);
                        END;
                        1: BEGIN
                           Prog1Bit(K);
                        END;
                     END;
                     Write('Program another? (Y): ');
                     Readln(Str);
                     IF Length(Str)=0 THEN Test := FALSE ELSE BEGIN
                        WHILE ((Str<>'Y') AND (Str<>'y') AND ( Str<> 'N')
                             AND (Str<>'n')) DO BEGIN
                           BadBeep;
                           Writeln('Please respond "Y" or "N".');
                           Readln (Str);
                        END;
                        Test := NOT ((Str = 'Y') OR (Str = 'y'));
                     END;
                  UNTIL Test;
               END ELSE Writeln('Data input error - copy aborted.');
            END;

            10: BEGIN
               Writeln('Executing Copy EPROM to EPROM - Type <ESC> to exit');
               write('Do you wish to copy the status bytes? (N): ');
               readln(ISt);
               AB := (ISt = 'Y') or (ISt = 'y');
               Writeln('Please Touch Source Device.');
               if AB then begin
                   repeat until TouchReset(K) or keypressed;
                   if not RdStatusRT then begin
                       AB := false;    BadBeep;
                       writeln('Fail Status Read!  Status Copy Aborted.');
                   end;
               end;
               CmdTyp:=$0F;
               Test:=TRUE;
               REPEAT
                  PGNM:=0;
                  BYCNT:=0;
                  Numtrys:=1;
                  E1:=0;
                  TA1:=0;
                  ReadSuccess:=FALSE;
                  Endloop:=FALSE;
                  Output:=FALSE;
                  IF TouchReset(K) THEN BEGIN
                     IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                     ReadEPROMcrc(K,Output,ReadSuccess);
                     IF Numtrys>20 THEN BEGIN
                        GetTime(Hour,Min2,Sec2,Hun2);
                        BadBeep;    AB := false;
                        Writeln;
                        Writeln('Unable to read after ',Numtrys- 1,
                             ' attempt(s).');
                        IF Diag THEN BEGIN
                           Writeln(E1,
                                ' CRC Errors Encountered during Reading.');
                           CalcTime;
                        END;
                        Writeln;
                     END;
                  END;
               UNTIL (ReadSuccess) OR (Numtrys>20) or KeyPressed;
               I := 0;
               REPEAT
                  IF TouchReset(K) THEN I := 0 ELSE Inc(I);
                             {terminates loop when probe}
               UNTIL (I = 100) or KeyPressed;
                             {is lifted from Touch Memory}
               IF ReadSuccess THEN BEGIN
                  REPEAT
                     Write('Please Touch Destination DS1982 - ');
                     TA1 := 0;
                     TA2 := 0;
                     NumByt:=128;
                     FOR I:=0 TO 127 DO Dbyte[I]:=BYT[I];
                     CASE Pass OF
                        8: BEGIN
                           Prog8Bit(K);
                        END;
                        4: BEGIN
                           Prog4Bit(K);
                        END;
                        2: BEGIN
                           Prog2Bit(K);
                        END;
                        1: BEGIN
                           Prog1Bit(K);
                        END;
                     END;
                     if AB then begin
                         for BC := 0 to 7 do begin
                             DByte[BC] := StByte[BC];
                         end;
                         repeat until TouchReset(K) or keypressed;
                         if not keypressed then begin
                             CmdTyp := $55;
                             TA1 := 0;
                             NumByt := 8;
                             case Pass of
                                 8: Prog8Bit(K);
                                 4: Prog4Bit(K);
                                 2: Prog2Bit(K);
                                 1: Prog1Bit(K);
                             end;
                         end else writeln('Status write aborted from keyboard.');
                     end;
                     Write('Program another? (Y): ');
                     Readln(Str);
                     IF Length(Str)=0 THEN Test := FALSE ELSE BEGIN
                        WHILE ((Str<>'Y') AND (Str<>'y') AND ( Str<> 'N')
                             AND (Str<>'n')) DO BEGIN
                           BadBeep;
                           Writeln('Please respond "Y" or "N".');
                           Readln (Str);
                        END;
                        Test := NOT ((Str='Y') OR (Str='y'));
                     END;
                  UNTIL Test;
               END ELSE Writeln('Data input error - copy aborted.');
            END;

            11: BEGIN
               Writeln('Executing Program Write Protect Bits');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write( 'Enter Page Number to Write Protect (0-3): ' );
                  Readln(Str);
                             {$R+}
                  Val(Str, L, N);
                             {$R-}
                  IF (N > 0) OR (L < 0) OR (L > 3) THEN BEGIN
                     BadBeep;
                     Writeln(
                         'Value for Page Number must be in the range 0-3.');
                  END ELSE BEGIN
                     PGNM:=L;
                     Test := TRUE;
                  END;
               END;
               BYCNT:=0;
               Numtrys:=1;
               E1:=0;
               TA1:=0;
               ReadSuccess:=FALSE;
               WriteLn('Touch device to write protect - Type <ESC> to exit');
               WHILE (NumTrys<21) AND NOT ReadSuccess AND NOT KeyPressed DO
               BEGIN
                  IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                  IF TouchReset(K) THEN BEGIN
                     WByte($CC);
                     Wbyte($AA);
                     WByte(TA1);
                     WByte(0);
                     CRCSav[4] := TouchByte($FF);
                     CRC8 := 0;
                     DOWCRC($AA);
                     DOWCRC(TA1);
                     DOWCRC(0);
                     IF CRC8 = CRCSav[4] THEN BEGIN
                        CRC8 := 0;
                        TA1:= 0;
                        FOR BYCNT := 0 TO 7 DO BEGIN
                           BYT[TA1] := RByte;
                           Inc(TA1);
                        END;
                        CRCSav[0] := TouchByte($FF);
                        IF CRCSav[0] = CRC8 THEN BEGIN
                           ReadSuccess := TRUE;
                           TA1:=0;
                           IF ((BYT[0] ShR PGNM) AND $01) = $01 THEN BEGIN
                              CmdTyp:=$55;
                              TA1:=0;
                              NumByt:=1;
                              Dbyte[0]:=(BYT[0]) AND (NOT(($01 ShL PGNM)));
                              CASE Pass OF
                                 8: BEGIN
                                    Prog8Bit(K);
                                 END;
                                 4: BEGIN
                                    Prog4Bit(K);
                                 END;
                                 2: BEGIN
                                    Prog2Bit(K);
                                 END;
                                 1: BEGIN
                                    Prog1Bit(K);
                                 END;
                              END;

                           END ELSE BEGIN
                              BadBeep;
                              Writeln('Page ',PGNM,
                                   ' is Already Write-Protected,' );
                           END;
                        END ELSE BEGIN
                           E1:=E1+1;
                           Numtrys:=Numtrys+1;
                        END;

                     END ELSE BEGIN
                        E1:=E1+1;
                        Numtrys:=Numtrys+1;
                     END;
                  END;
               END;
               IF Numtrys>20 THEN BEGIN
                  GetTime(Hour,Min2,Sec2,Hun2);
                  BadBeep;
                  Writeln('Unable to read after ',Numtrys-1,
                       ' attempt(s).');
                  IF Diag THEN BEGIN
                     Writeln(E1, ' CRC Errors Encountered during Reading.');
                     CalcTime;
                  END;
                  Writeln;
               END;
            END;

            12: BEGIN
               Writeln( 'Executing Program Page Redirection Bytes');
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write( 'Enter Number of Page to be Redirected (0-3): ' ) ;
                  Readln(Str);
                             {$R+}
                  Val(Str, L, N);
                             {$R-}
                  IF (N > 0) OR (L < 0) OR (L > 3) THEN BEGIN
                     BadBeep;
                     Writeln(
                         'Value for Page Number must be in the range 0-3.');
                  END ELSE BEGIN
                     PGNM:=L;
                     Test := TRUE;
                  END;
               END;
               Test := FALSE;
               WHILE NOT Test DO BEGIN
                  Write( 'Enter Number of Page to Redirect to (0-3): ');
                  Readln(Str);
                             {$R+}
                  Val(Str, L, N);
                             {$R-}
                  IF (N > 0) OR (L < 0) OR (L > 3) THEN BEGIN
                     BadBeep;
                     Writeln(
                         'Value for Page Number must be in the range 0-3.');
                  END ELSE BEGIN
                     IF L = PGNM THEN BEGIN
                        BadBeep;
                        Writeln( 'Page Cannot be Redirected to Itself.');
                     END ELSE BEGIN
                        RedPGNM:=L;
                        Test := TRUE;
                     END;
                  END;
               END;
               BYCNT:=0;
               Numtrys:=1;
               E1:=0;
               TA1:=1;
               ReadSuccess:=FALSE;
               WriteLn('Touch device to redirect - Type <ESC> to exit');
               WHILE (NumTrys<21) AND NOT ReadSuccess AND NOT KeyPressed DO
               BEGIN
                  IF Diag THEN GetTime(Hour,Min1,Sec1, Hun1);
                  IF TouchReset(K) THEN BEGIN
                     WByte($CC);
                     Wbyte($AA);
                     WByte(TA1);
                     WByte(0);
                     CRCSav[4] := TouchByte($FF);
                     CRC8 := 0;
                     DOWCRC($AA);
                     DOWCRC(TA1);
                     DOWCRC(0);
                     IF CRC8 = CRCSav[4] THEN BEGIN
                        CRC8 := 0;
                        TA1:= 1;
                        FOR BYCNT := 1 TO 7 DO BEGIN
                           BYT[TA1] := RByte;
                           Inc(TA1);
                        END;
                        CRCSav[0] := TouchByte($FF);
                        IF CRCSav[0] = CRC8 THEN BEGIN
                           ReadSuccess := TRUE;
                           IF BYT[PGNM+1] = $FF THEN BEGIN
                              CmdTyp:=$55;
                              TA1:=PGNM+1;
                              NumByt:=1;
                              Dbyte[TA1]:=NOT(RedPGNM);
                              CASE Pass OF
                                 8: BEGIN
                                    Prog8Bit(K);
                                 END;
                                 4: BEGIN
                                    Prog4Bit(K);
                                 END;
                                 2: BEGIN
                                    Prog2Bit(K);
                                 END;
                                 1: BEGIN
                                    Prog1Bit(K);
                                 END;
                              END;
                           END ELSE BEGIN
                              BadBeep;
                              Write('Page ',PGNM,
                                   ' is Already Redirected to Page ' );
                              Writeln(Hex(NOT BYT[PGNM+1]));
                           END;
                        END ELSE BEGIN
                           E1:=E1+1;
                           Numtrys:=Numtrys+1;
                        END;
                     END ELSE BEGIN
                        E1:=E1+1;
                        Numtrys:=Numtrys+1;
                     END;
                  END;
               END;
               IF Numtrys>20 THEN BEGIN
                  GetTime(Hour,Min2,Sec2,Hun2);
                  BadBeep;
                  Writeln('Unable to read after ',Numtrys-1,
                       ' attempt(s).');
                  IF Diag THEN BEGIN
                     Writeln(E1, ' CRC Errors Encountered during Reading.');
                     CalcTime;
                  END;
                  Writeln;
               END;
            END;

            13: BEGIN
               Writeln('Attach DS1982 and press <ENTER> to Calibrate Serial Port');
               Test := FALSE;
               readln;
               WHILE NOT Test DO BEGIN
                  Write('Enter value for TA1(0): ');
                  Readln(Str);
                  IF Length(Str)=0 THEN BEGIN
                     TA1Sav :=0;
                     Test := TRUE;
                  END ELSE BEGIN {$R+}
                     Val(Str, L, N);
                             {$R-}
                     IF (N > 0) OR (L < 0) OR (L > 120) THEN BEGIN
                        BadBeep;
                        Writeln(
                             'Value for TA1 must be in the range 0-120' );
                     END ELSE BEGIN
                        TA1Sav := L;
                        Test := TRUE;
                     END;
                  END;
               END;
               Writeln('Note: TA2 = 00h ');
               FindBlank(K,NotBlank,UnPro,UnProTA1);
               IF NOT NotBlank THEN BEGIN
                  TA1 := TA1Sav;
                  Dbyte[TA1Sav] := $00;
                  Dbyte[TA1Sav+1] := $00;
                  Pass := 8;
                  IF TouchReset(K) THEN BEGIN
                             {Attempt to program 8 bits at a time}
                     Writeln( 'Attempting to program 8 bits at a time.' );
                     WByte($CC);
                     Wbyte($0F);
                     WByte(TA1);
                     WByte(0);
                     WByte(Dbyte[TA1]);
                     X := TouchByte($FF);
                     CRC8 := 0;
                     DOWCRC($0F);
                     DOWCRC(TA1);
                     DOWCRC(0);
                     DOWCRC(Dbyte[TA1]);
                     IF (CRC8 = X) AND (Pass = 8) THEN BEGIN
                        PulWidth(Propw);
                             {Apply Program Pulse}
                        BYT[TA1] := TouchByte($FF);
                             {Read EPROM Data}
                        IF BYT[TA1] = Dbyte[TA1] THEN BEGIN
                           Inc (TA1);
                           CRC8 := TA1;
                           Wbyte(Dbyte[TA1]);
                           DOWCRC(Dbyte[TA1]);
                           X := TouchByte($FF);
                           PulWidth(Propw);
                             {Apply Program Pulse}
                           BYT[TA1] := TouchByte($FF);
                             {Read EPROM Data}
                           IF BYT[TA1] = Dbyte[TA1] THEN BEGIN
                              GoodBeep;
                              Write(
                         'Selected Serial Port is Capable of Programming ');
                              Writeln( '8 EPROM Bits at a Time.(B=8)');
                           END ELSE BEGIN
                              Write(
                     'Selected Serial Port is Not Capable of Programming ');
                              Writeln('8 EPROM Bits at a Time.');
                              Pass := 4;
                              BadBeep;
                           END;

                        END ELSE BEGIN
                           Write(
                     'Selected Serial Port is Not Capable of Programming ');
                           Writeln('8 EPROM Bits at a Time.');
                           Pass := 4;
                           BadBeep;
                        END;
                     END;
                  END;

                  M := 0;    {Attempt to program bytes in 2 nibbles}
                  Inc (TA1);
                  TA1Sav := TA1;
                  Dbyte[TA1Sav]   := $0F;
                  Dbyte[TA1Sav+1] := $0F;
                  IF Pass =4 THEN Writeln(
                       'Attempting to program 4 bits at a time.') ;
                  WHILE M < 2 DO BEGIN
                     IF TouchReset(K) THEN BEGIN
                        TA1 := TA1Sav;
                        WByte($CC);
                        Wbyte($0F);
                        WByte(TA1);
                        WByte(0);
                        WByte(Dbyte[TA1]);
                        X := TouchByte($FF);
                        CRC8 := 0;
                        DOWCRC($0F);
                        DOWCRC(TA1);
                        DOWCRC(0);
                        DOWCRC(Dbyte[TA1]);
                        IF (CRC8 = X) AND (Pass = 4) THEN BEGIN
                           PulWidth(Propw);         {Apply Program Pulse}
                           BYT[TA1] := TouchByte($FF);  {Read EPROM Data}
                           IF ((BYT[TA1] = Dbyte[TA1]) AND (M=0)) OR ((
                                BYT[TA1]=$00) AND (M=1)) THEN BEGIN
                              Inc (TA1);
                              CRC8 := TA1;
                              Wbyte(Dbyte[TA1]);
                              DOWCRC(Dbyte[TA1]);
                              X := TouchByte($FF);
                              PulWidth(Propw);        {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF); {Read EPROM Data}
                              IF (BYT[TA1] = $00) AND (M = 1) THEN BEGIN
                                 GoodBeep;
                                 Write(
                         'Selected Serial Port is Capable of Programming ');
                                 Writeln('4 EPROM Bits at a Time.(B=4)');
                              END ELSE IF (BYT[TA1] <> Dbyte[TA1])
                              AND (M= 0) OR (M=1) THEN BEGIN
                                 Write( 'Selected Serial Port is Not ' );
                                 Write('Capable of Programming ') ;
                                 Writeln( '4 EPROM Bits at a Time.') ;
                                 Pass := 2;
                                 Inc(M);
                                 BadBeep;
                              END;

                           END ELSE BEGIN
                              Write( 'Selected Serial Port is Not') ;
                              Write(' Capable of Programming ');
                              Writeln('4 EPROM Bits at a Time.');
                              Pass := 2;
                              Inc(M);
                              BadBeep;
                           END;
                        END;
                     END;
                     Inc (M);
                     Dbyte[TA1Sav] := $F0;
                     Dbyte[TA1Sav+1] := $F0;
                  END;

                  M := 0;    {Attempt to program a pair at a time}
                  Inc (TA1);
                  TA1Sav := TA1;
                  Dbyte[TA1Sav] := $3F;
                  Dbyte[TA1Sav+1] := $3F;
                  IF Pass = 2 THEN Writeln(
                       'Attempting to program 2 bits at a time.') ;
                  WHILE M < 4 DO BEGIN
                     IF TouchReset(K) THEN BEGIN
                        TA1 := TA1Sav;
                        WByte($CC);
                        Wbyte($0F);
                        WByte(TA1);
                        WByte(0);
                        WByte(Dbyte[TA1]);
                        X := TouchByte($FF);
                        CRC8 := 0;
                        DOWCRC($0F);
                        DOWCRC(TA1);
                        DOWCRC(0);
                        DOWCRC(Dbyte[TA1]);
                        IF (CRC8 = X) AND (Pass = 2) THEN BEGIN
                           PulWidth(Propw);         {Apply Program Pulse}
                           BYT[TA1] := TouchByte($FF);  {Read EPROM Data}
                           IF BYT[TA1] = ((($3F) ShR (M*2))AND Dbyte[ TA1]
                                ) THEN BEGIN
                              Inc (TA1);
                              CRC8 := TA1;
                              Wbyte(Dbyte[TA1]);
                              DOWCRC(Dbyte[TA1]);
                              X := TouchByte($FF);
                              PulWidth(Propw);        {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF); {Read EPROM Data}
                              IF (BYT[TA1] = $00) AND (M = 3) THEN BEGIN
                                 GoodBeep;
                                 Write(
                         'Selected Serial Port is Capable of Programming ');
                                 Writeln(
                                      '2 EPROM Bits at a Time.(B=2)');
                              END ELSE IF BYT[TA1] <> ((($3F) ShR (M* 2))
                                   AND Dbyte[TA1]) THEN BEGIN
                                 Write( 'Selected Serial Port is Not');
                                 Write( ' Capable of Programming ') ;
                                 Writeln( '2 EPROM Bits at a Time.') ;
                                 Pass := 1;
                                 M:=4;
                                 BadBeep;
                              END;

                           END ELSE BEGIN
                              Write( 'Selected Serial Port is Not') ;
                              Write(' Capable of Programming ');
                              Writeln('2 EPROM Bits at a Time.');
                              Pass := 1;
                              M:=4;
                              BadBeep;
                           END;
                        END;
                     END;
                     Inc (M);
                     Dbyte[TA1Sav] := NOT ((NOT(Dbyte[TA1Sav])) ShR 2);
                     Dbyte[TA1Sav+1] := NOT ((NOT(Dbyte[TA1Sav+1] )) ShR 2);
                  END;

                  M := 0;    {Attempt to program a bit at a time}
                  Inc (TA1);
                  TA1Sav := TA1;
                  Dbyte[TA1Sav] := $7F;
                  Dbyte[TA1Sav+1] := $7F;
                  IF Pass = 1 THEN Writeln(
                       'Attempting to program 1 bit at a time.');
                  WHILE M < 8 DO BEGIN
                     IF TouchReset(K) THEN BEGIN
                        TA1 := TA1Sav;
                        WByte($CC);
                        Wbyte($0F);
                        WByte(TA1);
                        WByte(0);
                        WByte(Dbyte[TA1]);
                        X := TouchByte($FF);
                        CRC8 := 0;
                        DOWCRC($0F);
                        DOWCRC(TA1);
                        DOWCRC(0);
                        DOWCRC(Dbyte[TA1]);
                        IF (CRC8 = X) AND (Pass = 1) THEN BEGIN
                           PulWidth(Propw);         {Apply Program Pulse}
                           BYT[TA1] := TouchByte($FF);  {Read EPROM Data}
                           IF BYT[TA1] = ((($7F) ShR (M))AND Dbyte[ TA1])
                                THEN BEGIN
                              Inc (TA1);
                              CRC8 := TA1;
                              Wbyte(Dbyte[TA1]);
                              DOWCRC(Dbyte[TA1]);
                              X := TouchByte($FF);
                              PulWidth(Propw);        {Apply Program Pulse}
                              BYT[TA1] := TouchByte($FF); {Read EPROM Data}
                              IF (BYT[TA1] = $00) AND (M = 7) THEN BEGIN
                                 GoodBeep;
                                 Write(
                         'Selected Serial Port is Capable of Programming ');
                                 Writeln( '1 EPROM Bit at a Time.(B=1)');
                              END ELSE IF BYT[TA1] <> ((($7F) ShR (M))
                                   AND Dbyte[TA1]) THEN BEGIN
                                 Write( 'Selected Serial Port is Not');
                                 Write( ' Capable of Programming ');
                                 Writeln( '1 EPROM Bit at a Time.');
                                 M:=8;
                                 BadBeep;
                              END;

                           END ELSE BEGIN
                              Write( 'Selected Serial Port is Not');
                              Write(' Capable of Programming ');
                              Writeln('1 EPROM Bit at a Time.');
                              M:=8;
                              BadBeep;
                           END;
                        END;
                     END;
                     Inc (M);
                     Dbyte[TA1Sav]   := NOT ((NOT(Dbyte[TA1Sav])) ShR 1);
                     Dbyte[TA1Sav+1] := NOT ((NOT(Dbyte[TA1Sav+1] )) ShR 1);
                  END;
               END;
            END;

         ELSE
            Writeln( 'Please select a value from the menu choices.');
            Badbeep;
         END;
      UNTIL B = 0;
      Writeln('Program Exited.');

   END ELSE Writeln('COM', K, ' DOES NOT EXIST');
END.
